@@ -1,5 +1,5 @@
 /*
- *  $Id: file.c 28280 2025-07-24 16:22:16Z yeti-dn $
+ *  $Id: file.c 28776 2025-11-03 20:13:17Z klapetek $
  *  Copyright (C) 2003-2022 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -421,10 +421,11 @@ gwy_app_file_stack(void)
     gchar *fname_sys;
     gchar *s;// *name;
     gint col, row, response, chsel = 0;
-    gint newid, *ids, *chs, n, i, j, nids = 0, lnids, nchs, nchan;
+    gint newid, *ids, *chs, n, i, j, k, nids = 0, lnids, nchs, nchan;
     gchar **namelist = NULL;
     gchar **channamelist = NULL;
     gint *count = NULL;
+    gint nsames;
     gint xres = 0, yres = 0, nlayers;
     gdouble xreal = 0, yreal = 0, *bdata, *ddata;
     FileStackError serr = STACK_OK;
@@ -508,7 +509,7 @@ gwy_app_file_stack(void)
     /* set the channels to display to user to pick */
     nchan = 0;
     for (i = 0; i < nids; i++) {
-        if (count[i] == n)
+        if (count[i] >= n)
             nchan++;
     }
     nlayers = n;
@@ -516,7 +517,7 @@ gwy_app_file_stack(void)
     channamelist = g_new(gchar*, nchan);
     nchan = 0;
     for (i = 0; i < nids; i++) {
-        if (count[i] == n)
+        if (count[i] >= n)
             channamelist[nchan++] = g_strdup(namelist[i]);
     }
 
@@ -577,9 +578,12 @@ gwy_app_file_stack(void)
                    nchs++;
                while (chs[nchs] != -1);
 
+               nsames = 0;
                if (nchs != 1) {
-                  serr = STACK_ERROR_MULTIPLE;
-                  break;
+                   for (k = 0; k < chsel; k++)
+                       if (strcmp(channamelist[chsel], channamelist[k]) == 0) {
+                           nsames++;
+                       }
                }
 
                if (!serr && n == 0) {
@@ -598,7 +602,7 @@ gwy_app_file_stack(void)
                    gwy_brick_set_si_unit_w(brick, gwy_data_field_get_si_unit_z(dfield0));
                }
                else {
-                   dfield = gwy_container_get_object(data, gwy_app_get_data_key_for_id(chs[0]));
+                   dfield = gwy_container_get_object(data, gwy_app_get_data_key_for_id(chs[0 + nsames]));
                    if (gwy_data_field_check_compatibility(dfield0, dfield,
                                                           GWY_DATA_COMPATIBILITY_RES | GWY_DATA_COMPATIBILITY_REAL
                                                           | GWY_DATA_COMPATIBILITY_LATERAL
