@@ -1,6 +1,6 @@
 /*
- *  $Id: meta.c 27997 2025-05-16 16:57:51Z yeti-dn $
- *  Copyright (C) 2003-2021 David Necas (Yeti), Petr Klapetek.
+ *  $Id: meta.c 28683 2025-10-21 09:23:02Z yeti-dn $
+ *  Copyright (C) 2003-2025 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
@@ -52,6 +52,7 @@ typedef struct {
     GwyContainer *meta;
     gulong changed_id;
     gulong destroy_id;
+    GtkListStore *store;
     GtkWidget *window;
     GtkWidget *user_filter;
     GtkWidget *treeview;
@@ -91,7 +92,7 @@ static void             gwy_meta_new_item             (MetadataBrowser *browser)
 static void             gwy_meta_delete_item          (MetadataBrowser *browser);
 static void             gwy_meta_destroy              (MetadataBrowser *browser);
 static void             gwy_meta_data_finalized       (MetadataBrowser *browser);
-static void             gwy_meta_focus_iter           (MetadataBrowser *browser,
+static void             gwy_meta_focus_store_iter     (MetadataBrowser *browser,
                                                        GtkTreeIter *iter);
 static gboolean         gwy_meta_find_key             (MetadataBrowser *browser,
                                                        GQuark quark,
@@ -418,6 +419,7 @@ gwy_meta_switch_data(MetadataBrowser *browser,
         gwy_container_pass_object(data, quark, meta);
         store = gtk_list_store_new(1, G_TYPE_UINT);
     }
+    browser->store = store;
 
     GWY_SIGNAL_HANDLER_DISCONNECT(browser->meta, browser->changed_id);
     if (browser->meta) {
@@ -435,7 +437,6 @@ gwy_meta_switch_data(MetadataBrowser *browser,
                                            gwy_meta_filter_visible_func,
                                            browser,
                                            NULL);
-    g_object_unref(store);
 
     treeview = GTK_TREE_VIEW(browser->treeview);
     gtk_tree_view_set_model(treeview, GTK_TREE_MODEL(filtered_store));
@@ -623,15 +624,13 @@ gwy_meta_item_changed(GwyContainer *container,
                       MetadataBrowser *browser)
 {
     GtkListStore *store;
-    GtkTreeView *treeview;
     GtkTreeIter iter;
     G_GNUC_UNUSED const gchar *key;
 
     key = g_quark_to_string(quark);
     gwy_debug("Meta item <%s> changed", key);
     g_return_if_fail(quark);
-    treeview = GTK_TREE_VIEW(browser->treeview);
-    store = GTK_LIST_STORE(gtk_tree_view_get_model(treeview));
+    store = browser->store;
 
     if (gwy_meta_find_key(browser, quark, &iter)) {
         if (gwy_container_contains(container, quark))
@@ -642,7 +641,7 @@ gwy_meta_item_changed(GwyContainer *container,
     }
 
     gtk_list_store_insert_with_values(store, &iter, G_MAXINT, META_KEY, quark, -1);
-    gwy_meta_focus_iter(browser, &iter);
+    gwy_meta_focus_store_iter(browser, &iter);
 }
 
 static void
@@ -665,7 +664,7 @@ gwy_meta_new_item(MetadataBrowser *browser)
 
     if (gwy_container_contains(browser->meta, quark)) {
         if (gwy_meta_find_key(browser, quark, &iter))
-            gwy_meta_focus_iter(browser, &iter);
+            gwy_meta_focus_store_iter(browser, &iter);
     }
     else {
         if (g_random_int() % 4 == 0)
@@ -700,10 +699,14 @@ gwy_meta_delete_item(MetadataBrowser *browser)
 static void
 gwy_meta_destroy(MetadataBrowser *browser)
 {
-    g_free(browser->filter_str);
+    GWY_FREE(browser->filter_str);
+    GWY_OBJECT_UNREF(browser->store);
     GWY_SIGNAL_HANDLER_DISCONNECT(browser->meta, browser->changed_id);
-    g_object_set_data(G_OBJECT(browser->meta), "metadata-browser", NULL);
-    g_object_weak_unref(G_OBJECT(browser->meta), (GWeakNotify)&gwy_meta_data_finalized, browser);
+    if (browser->meta) {
+        g_object_set_data(G_OBJECT(browser->meta), "metadata-browser", NULL);
+        g_object_weak_unref(G_OBJECT(browser->meta), (GWeakNotify)&gwy_meta_data_finalized, browser);
+        browser->meta = NULL;
+    }
     g_free(browser);
 }
 
@@ -717,15 +720,20 @@ gwy_meta_data_finalized(MetadataBrowser *browser)
 }
 
 static void
-gwy_meta_focus_iter(MetadataBrowser *browser,
-                    GtkTreeIter *iter)
+gwy_meta_focus_store_iter(MetadataBrowser *browser,
+                          GtkTreeIter *iter)
 {
     GtkTreeView *treeview;
     GtkTreeViewColumn *column;
     GtkTreePath *path;
+    GtkTreeIter filter_iter;
+    GtkTreeModel *model;
 
     treeview = GTK_TREE_VIEW(browser->treeview);
-    path = gtk_tree_model_get_path(gtk_tree_view_get_model(treeview), iter);
+    model = gtk_tree_view_get_model(treeview);
+    if (!gtk_tree_model_filter_convert_child_iter_to_iter(GTK_TREE_MODEL_FILTER(model), &filter_iter, iter))
+        return;
+    path = gtk_tree_model_get_path(model, &filter_iter);
     gtk_tree_view_scroll_to_cell(treeview, path, NULL, FALSE, 0.0, 0.0);
     column = gtk_tree_view_get_column(treeview, META_KEY);
     gtk_tree_view_set_cursor(treeview, path, column, FALSE);
@@ -738,10 +746,9 @@ gwy_meta_find_key(MetadataBrowser *browser,
                   GQuark quark,
                   GtkTreeIter *iter)
 {
-    GtkTreeModel *model;
+    GtkTreeModel *model = GTK_TREE_MODEL(browser->store);
     GQuark q;
 
-    model = gtk_tree_view_get_model(GTK_TREE_VIEW(browser->treeview));
     if (gtk_tree_model_get_iter_first(model, iter)) {
         do {
             gtk_tree_model_get(model, iter, META_KEY, &q, -1);
