@@ -1,5 +1,5 @@
 /*
- *  $Id: param-def.c 27437 2025-03-04 14:26:52Z yeti-dn $
+ *  $Id: param-def.c 28708 2025-10-22 08:27:45Z yeti-dn $
  *  Copyright (C) 2021-2023 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -1599,7 +1599,7 @@ _gwy_param_def_use(GwyParamDef *pardef, GwyParams *params)
 #ifdef DEBUG_USERS
     /* When presets are implemented using GwyParams then multiple GwyParams instances created for one GwyParamDef
      * are common.  Keep enabled this check during development but we may want a mechanism to exclude presets. */
-    if (!g_slist_find(priv->users, params)) {
+    if (params && !g_slist_find(priv->users, params)) {
         /* XXX: Hardcoded warning suppression for known modules with GwyParamResource presets and internal
          * parameter set duplication. */
         if (priv->users && !gwy_stramong(priv->function_name,
