@@ -1,5 +1,5 @@
 /*
- *  $Id: params.c 25576 2023-07-28 11:59:57Z yeti-dn $
+ *  $Id: params.c 28707 2025-10-22 08:21:19Z yeti-dn $
  *  Copyright (C) 2021-2022 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -1751,7 +1751,8 @@ set_int_value(GwyParamValue *pvalue, const GwyParamDefItem *def, gint value)
     gint goodvalue = _gwy_param_def_rectify_int(def, value);
 
     if (value != goodvalue) {
-        g_warning("Value %d is not in the range of int id=%d.", value, def->id);
+        g_warning("Value %d is not in the range of int id=%d (%s).",
+                  value, def->id, def->name ? def->name : "NULL");
     }
     if (pvalue->i == goodvalue)
         return FALSE;
@@ -1765,7 +1766,8 @@ set_random_seed_value(GwyParamValue *pvalue, const GwyParamDefItem *def, gint va
     gint goodvalue = _gwy_param_def_rectify_random_seed(def, value);
 
     if (value != goodvalue) {
-        g_warning("Value %d is not in the range of random seed id=%d.", value, def->id);
+        g_warning("Value %d is not in the range of random seed id=%d (%s).",
+                  value, def->id, def->name ? def->name : "NULL");
     }
     if (pvalue->i == goodvalue)
         return FALSE;
@@ -1779,7 +1781,8 @@ set_double_value(GwyParamValue *pvalue, const GwyParamDefItem *def, gdouble valu
     gdouble goodvalue = _gwy_param_def_rectify_double(def, value);
 
     if (fabs(value - goodvalue) > 1e-14*(fabs(value) + fabs(goodvalue))) {
-        g_warning("Value %.14g is not in the range of double id=%d.", value, def->id);
+        g_warning("Value %.14g is not in the range of double id=%d (%s).",
+                  value, def->id, def->name ? def->name : "NULL");
     }
     if (pvalue->d == goodvalue)
         return FALSE;
@@ -1794,7 +1797,8 @@ set_enum_value(GwyParamValue *pvalue, const GwyParamDefItem *def, gint value)
 
     goodvalue = _gwy_param_def_rectify_enum(def, value);
     if (value != goodvalue) {
-        g_warning("Value %d is not in enum id=%d.", value, def->id);
+        g_warning("Value %d is not in enum id=%d (%s).",
+                  value, def->id, def->name ? def->name : "NULL");
     }
     if (pvalue->i == goodvalue)
         return FALSE;
@@ -1809,7 +1813,8 @@ set_flags_value(GwyParamValue *pvalue, const GwyParamDefItem *def, guint value)
 
     goodvalue = _gwy_param_def_rectify_flags(def, value);
     if (value != goodvalue) {
-        g_warning("Value %u is not in flags id=%d.", value, def->id);
+        g_warning("Value %u is not in flags id=%d (%s).",
+                  value, def->id, def->name ? def->name : "NULL");
     }
     if (pvalue->u == goodvalue)
         return FALSE;
@@ -1824,7 +1829,8 @@ set_report_type_value(GwyParamValue *pvalue, const GwyParamDefItem *def, GwyResu
 
     goodvalue = _gwy_param_def_rectify_report_type(def, value);
     if (value != goodvalue) {
-        g_warning("Value %u is not in report type id=%d.", value, def->id);
+        g_warning("Value %u is not in report type id=%d (%s).",
+                  value, def->id, def->name ? def->name : "NULL");
     }
     if (pvalue->rt == goodvalue)
         return FALSE;
@@ -1859,7 +1865,8 @@ set_curve_int_value(GwyParamValue *pvalue, const GwyParamDefItem *def, gint valu
 {
     gint goodvalue = MAX(value, 0);
     if (value != goodvalue) {
-        g_warning("Value %d is not a valid curve number in id=%d.", value, def->id);
+        g_warning("Value %d is not a valid curve number in id=%d (%s).",
+                  value, def->id, def->name ? def->name : "NULL");
         return FALSE;
     }
     pvalue->cu.use_string = FALSE;
@@ -1903,10 +1910,12 @@ set_color_value(GwyParamValue *pvalue, const GwyParamDefItem *def, GwyRGBA value
     GwyRGBA *rgba;
 
     if (value.r != goodvalue.r || value.g != goodvalue.g || value.b != goodvalue.b) {
-        g_warning("Color component values are not in the allowed range of color id=%d.", def->id);
+        g_warning("Color component values (%g, %g, %g) are not in the allowed range of color id=%d (%s).",
+                  value.r, value.g, value.b, def->id, def->name ? def->name : "NULL");
     }
     if (def->def.c.has_alpha && value.a != goodvalue.a) {
-        g_warning("Alpha value is not in the allowed range of color id=%d.", def->id);
+        g_warning("Alpha value %g is not in the allowed range of color id=%d (%s).",
+                  value.a, def->id, def->name ? def->name : "NULL");
     }
     rgba = &pvalue->c;
     if (rgba->r == goodvalue.r && rgba->g == goodvalue.b && rgba->b == goodvalue.b && rgba->a == goodvalue.a)
