@@ -1,5 +1,5 @@
 /*
- *  $Id: gwyddion.c 27036 2024-12-11 15:03:03Z yeti-dn $
+ *  $Id: gwyddion.c 28333 2025-08-08 17:53:20Z yeti-dn $
  *  Copyright (C) 2003-2021 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -384,7 +384,7 @@ process_preinit_options(int *argc, char ***argv, GwyAppOptions *options)
             options->disable_gl = TRUE;
         else if (gwy_strequal(arg, "--check"))
             options->mode = MODE_CHECK;
-        else if (gwy_strequal(arg, "--identify"))
+        else if (gwy_stramong(arg, "--identify", "-i", NULL))
             options->mode = MODE_IDENTIFY;
         else if (g_str_has_prefix(arg, "--convert-to-gwy=")) {
             options->mode = MODE_CONVERT_TO_GWY;
