@@ -1,5 +1,5 @@
 /*
- *  $Id: gwysiunit.c 26095 2024-01-04 13:53:02Z yeti-dn $
+ *  $Id: gwysiunit.c 28480 2025-09-02 14:49:00Z yeti-dn $
  *  Copyright (C) 2004-2022 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -795,11 +795,9 @@ fix_unit_name(GString *str)
         { 3, "",    "arb",     },
         { 3, "µm",  "mum",     },
         { 4, "",    "a.u.",    },
-        { 5, "",    "a. u.",   },
         { 6, "",    "counts",  },
         { 6, "µm",  "micron",  },
         { 7, "µm",  "microns", },
-        { 7, "µm",  "micro m", },
     };
 
     /* Long names and their prefixes. */
@@ -1006,6 +1004,9 @@ gwy_si_unit_parse(GwySIUnit *siunit,
 
     buf = g_string_new(NULL);
 
+    if (gwy_strequal(string, "a. u."))
+        string += strlen(string);
+
     /* the rest are units */
     while (*string) {
         /* here we actually parse the number, also after a division sign */
