@@ -1,5 +1,5 @@
 /*
- *  $Id: brick.h 23262 2021-03-15 14:11:31Z yeti-dn $
+ *  $Id: brick.h 29010 2025-12-16 12:15:41Z klapetek $
  *  Copyright (C) 2003-2019 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -355,8 +355,14 @@ void              gwy_brick_set_xy_plane      (GwyBrick *brick,
                                                gint lev);
 void              gwy_brick_add_to_xy_planes  (GwyBrick *brick,
                                                GwyDataField *plane);
+void              gwy_brick_multiply_xy_planes(GwyBrick *brick,
+                                               GwyDataField *plane);
 void              gwy_brick_add_to_z_lines    (GwyBrick *brick,
                                                GwyDataLine *line);
+gint              gwy_brick_threshold         (GwyBrick *brick,
+                                               gdouble threshval,
+                                               gdouble bottom,
+                                               gdouble top);
 
 G_END_DECLS
 
