@@ -1,5 +1,5 @@
 /*
- *  $Id: lawn.c 28051 2025-06-02 11:42:35Z yeti-dn $
+ *  $Id: lawn.c 28357 2025-08-12 19:33:35Z klapetek $
  *  Copyright (C) 2021 David Necas (Yeti), Petr Klapetek, Radek Slesinger.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -300,11 +300,12 @@ gwy_lawn_serialize(GObject *obj,
     GwyLawn *lawn;
     GwyLawnPrivate *priv;
     guint32 datasize;
-    gpointer pxoff, pyoff, pxyunit, *pcurveunits;
+    gpointer pxoff, pyoff, pxyunit;
     gint i, j;
     guint32 npoints, bsize, segdatasize;
     gdouble *alldata, *wrk;
-    gboolean any_units;
+    gboolean any_units = FALSE;
+    GwySIUnit **curveunits, *empty_unit = NULL;
     GByteArray *retval;
 
     lawn = GWY_LAWN(obj);
@@ -314,15 +315,18 @@ gwy_lawn_serialize(GObject *obj,
     pxyunit = unit_pointer_if_nonempty(lawn->si_unit_xy);
 
     priv = (GwyLawnPrivate*)lawn->priv;
-    pcurveunits = g_new(gpointer, priv->ncurves);
-    any_units = FALSE;
+    curveunits = g_new(GwySIUnit*, priv->ncurves);
+    empty_unit = gwy_si_unit_new(NULL);
     for (i = 0; i < priv->ncurves; i++) {
-        pcurveunits[i] = unit_pointer_if_nonempty(priv->si_units_curves[i]);
-        if (pcurveunits[i])
+        if (priv->si_units_curves[i] && !gwy_si_unit_equal_string(priv->si_units_curves[i], NULL)) {
             any_units = TRUE;
+            curveunits[i] = priv->si_units_curves[i];
+        }
+        else
+            curveunits[i] = empty_unit;
     }
     if (!any_units)
-        GWY_FREE(pcurveunits);
+        GWY_FREE(curveunits);
 
     npoints = lawn->xres * lawn->yres;
     datasize = 0;
@@ -355,7 +359,7 @@ gwy_lawn_serialize(GObject *obj,
             { 'i', "ncurves", &(priv->ncurves), NULL, },
             { 'I', "curvelengths", &(priv->curvelengths), &npoints, },
             { 'D', "data", &alldata, &datasize, },
-            { 'O', "si_units_curves", pcurveunits, &(priv->ncurves), },
+            { 'O', "si_units_curves", curveunits ? &curveunits : NULL, &(priv->ncurves), },
             { 'S', "curve_labels", &(priv->curvelabels), &(priv->ncurves), },
 
             { 'i', "nsegments", &(priv->nsegments), NULL, },
@@ -370,7 +374,8 @@ gwy_lawn_serialize(GObject *obj,
         }
 
         retval = gwy_serialize_pack_object_struct(buffer, GWY_LAWN_TYPE_NAME, j, spec);
-        g_free(pcurveunits);
+        g_object_unref(empty_unit);
+        g_free(curveunits);
         g_free(alldata);
 
         return retval;
@@ -383,29 +388,33 @@ gwy_lawn_get_size(GObject *obj)
     GwyLawn *lawn;
     GwyLawnPrivate *priv;
     guint32 datasize;
-    gpointer pxoff, pyoff, pxyunit, *pcurveunits;
+    gpointer pxoff, pyoff, pxyunit;
     gint i, j;
     gdouble *alldata;
     guint32 npoints, segdatasize;
-    gboolean any_units;
+    gboolean any_units = FALSE;
+    GwySIUnit **curveunits, *empty_unit = NULL;
     gsize retval;
 
     lawn = GWY_LAWN(obj);
-    priv = (GwyLawnPrivate*)lawn->priv;
 
     pxoff = lawn->xoff ? &lawn->xoff : NULL;
     pyoff = lawn->yoff ? &lawn->yoff : NULL;
     pxyunit = unit_pointer_if_nonempty(lawn->si_unit_xy);
 
-    pcurveunits = g_new(gpointer, priv->ncurves);
-    any_units = FALSE;
+    priv = (GwyLawnPrivate*)lawn->priv;
+    curveunits = g_new(GwySIUnit*, priv->ncurves);
+    empty_unit = gwy_si_unit_new(NULL);
     for (i = 0; i < priv->ncurves; i++) {
-        pcurveunits[i] = unit_pointer_if_nonempty(priv->si_units_curves[i]);
-        if (pcurveunits[i])
+        if (priv->si_units_curves[i] && !gwy_si_unit_equal_string(priv->si_units_curves[i], NULL)) {
             any_units = TRUE;
+            curveunits[i] = priv->si_units_curves[i];
+        }
+        else
+            curveunits[i] = empty_unit;
     }
     if (!any_units)
-        GWY_FREE(pcurveunits);
+        GWY_FREE(curveunits);
 
     npoints = lawn->xres * lawn->yres;
     datasize = 0;
@@ -431,7 +440,7 @@ gwy_lawn_get_size(GObject *obj)
             { 'i', "ncurves", &(priv->ncurves), NULL, },
             { 'I', "curvelengths", &(priv->curvelengths), &npoints, },
             { 'D', "data", &alldata, &datasize, },
-            { 'O', "si_units_curves", pcurveunits, &(priv->ncurves), },
+            { 'O', "si_units_curves", curveunits ? &curveunits : NULL, &(priv->ncurves), },
             { 'S', "curve_labels", &(priv->curvelabels), &(priv->ncurves), },
 
             { 'i', "nsegments", &(priv->nsegments), NULL, },
@@ -447,7 +456,8 @@ gwy_lawn_get_size(GObject *obj)
 
         retval = gwy_serialize_get_struct_size(GWY_LAWN_TYPE_NAME, j, spec);
 
-        g_free(pcurveunits);
+        g_object_unref(empty_unit);
+        g_free(curveunits);
         g_free(alldata);
 
         return retval;
@@ -1265,8 +1275,8 @@ gwy_lawn_get_value_format_curve(GwyLawn *lawn,
 
     for (i = 0; i < lawn->xres * lawn->yres; i++) {
         if (priv->curvedata[i]) {
-            for (j = 0; j < priv->curvelengths[j]; j++) {
-                idx = n * lawn->xres + j;
+            for (j = 0; j < priv->curvelengths[i]; j++) {
+                idx = n * priv->curvelengths[i] + j;
 
                 if (priv->curvedata[i][idx] > max)
                     max = priv->curvedata[i][idx];
