@@ -15,14 +15,18 @@ GWY_MODULE_QUERY(module_info)
 GwyModuleInfo* _gwy_module_query__cmap_asciiexport(void);
 GwyModuleInfo* _gwy_module_query__cmap_align(void);
 GwyModuleInfo* _gwy_module_query__cmap_basicops(void);
+GwyModuleInfo* _gwy_module_query__cmap_classify(void);
 GwyModuleInfo* _gwy_module_query__cmap_crop(void);
+GwyModuleInfo* _gwy_module_query__cmap_curvefilter(void);
 GwyModuleInfo* _gwy_module_query__cmap_cutter(void);
 GwyModuleInfo* _gwy_module_query__cmap_extractcurve(void);
 GwyModuleInfo* _gwy_module_query__cmap_fdfit(void);
+GwyModuleInfo* _gwy_module_query__cmap_filter(void);
 GwyModuleInfo* _gwy_module_query__cmap_fit(void);
 GwyModuleInfo* _gwy_module_query__cmap_fztofd(void);
 GwyModuleInfo* _gwy_module_query__cmap_linestat(void);
 GwyModuleInfo* _gwy_module_query__cmap_lockin(void);
+GwyModuleInfo* _gwy_module_query__cmap_peaks(void);
 GwyModuleInfo* _gwy_module_query__cmap_polylevel(void);
 GwyModuleInfo* _gwy_module_query__cmap_possearch(void);
 GwyModuleInfo* _gwy_module_query__cmap_simplemech(void);
@@ -33,14 +37,18 @@ static const GwyModuleRecord modules[] = {
   { _gwy_module_query__cmap_asciiexport, "cmap_asciiexport", },
   { _gwy_module_query__cmap_align, "cmap_align", },
   { _gwy_module_query__cmap_basicops, "cmap_basicops", },
+  { _gwy_module_query__cmap_classify, "cmap_classify", },
   { _gwy_module_query__cmap_crop, "cmap_crop", },
+  { _gwy_module_query__cmap_curvefilter, "cmap_curvefilter", },
   { _gwy_module_query__cmap_cutter, "cmap_cutter", },
   { _gwy_module_query__cmap_extractcurve, "cmap_extractcurve", },
   { _gwy_module_query__cmap_fdfit, "cmap_fdfit", },
+  { _gwy_module_query__cmap_filter, "cmap_filter", },
   { _gwy_module_query__cmap_fit, "cmap_fit", },
   { _gwy_module_query__cmap_fztofd, "cmap_fztofd", },
   { _gwy_module_query__cmap_linestat, "cmap_linestat", },
   { _gwy_module_query__cmap_lockin, "cmap_lockin", },
+  { _gwy_module_query__cmap_peaks, "cmap_peaks", },
   { _gwy_module_query__cmap_polylevel, "cmap_polylevel", },
   { _gwy_module_query__cmap_possearch, "cmap_possearch", },
   { _gwy_module_query__cmap_simplemech, "cmap_simplemech", },
