@@ -1,5 +1,5 @@
 /*
- *  $Id: cmap_fztofd.c 26949 2024-12-03 10:42:17Z yeti-dn $
+ *  $Id: cmap_fztofd.c 28887 2025-11-20 11:22:08Z klapetek $
  *  Copyright (C) 2021 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -178,6 +178,7 @@ fztofd(GwyContainer *data, GwyRunType runtype)
     GwyLawn *lawn = NULL;
     GwyDialogOutcome outcome = GWY_DIALOG_PROCEED;
     gint id;
+    gchar newtitle[100];
 
     g_return_if_fail(runtype & RUN_MODES);
     g_return_if_fail(g_type_from_name("GwyLayerPoint"));
@@ -208,7 +209,10 @@ fztofd(GwyContainer *data, GwyRunType runtype)
     if (outcome != GWY_DIALOG_HAVE_RESULT)
         execute(&args);
 
-    /* TODO: Add log, maybe more. */
+    g_snprintf(newtitle, sizeof(newtitle), "%s: FD", gwy_app_get_lawn_title(data, id));
+    gwy_app_set_lawn_title(data, id, newtitle);
+
+    gwy_app_curve_map_log_add(data, -1, id, "cmap::cmap_fztofd", NULL);
 
 end:
     GWY_OBJECT_UNREF(args.result);
