@@ -1,5 +1,5 @@
 /*
- *  $Id: cmap_polylevel.c 26949 2024-12-03 10:42:17Z yeti-dn $
+ *  $Id: cmap_polylevel.c 29034 2025-12-22 18:51:03Z yeti-dn $
  *  Copyright (C) 2021 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -37,6 +37,7 @@
 
 enum {
     PREVIEW_SIZE = 360,
+    MAXFITPARAMS = 6,
 };
 
 enum {
@@ -49,6 +50,8 @@ enum {
     PARAM_XPOS,
     PARAM_YPOS,
     PARAM_DEGREE,
+    PARAM_MEAN_CURVE,
+    PARAM_APPLY_TO_ALL,
 };
 
 typedef struct {
@@ -57,6 +60,7 @@ typedef struct {
     GwyLawn *result;
     GwyDataField *field;
     gint nsegments;
+    gdouble *meanfitparams;
 } ModuleArgs;
 
 typedef struct {
@@ -68,6 +72,7 @@ typedef struct {
     GwySelection *selection;
     GwySelection *graph_selection;
     GwyGraphModel *gmodel;
+    gboolean meanfitdone;
 } ModuleGUI;
 
 static gboolean         module_register         (void);
@@ -102,14 +107,20 @@ static void             do_polylevel            (const gdouble *xdata,
                                                  const gint *segments,
                                                  gint segment,
                                                  gboolean segment_enabled,
+                                                 gboolean apply_to_all,
                                                  gdouble from,
                                                  gdouble to,
                                                  gint degree,
                                                  gboolean subtract,
                                                  gdouble *retparam);
-
+static void             graph_selected          (GwySelection* selection,
+                                                 gint i,
+                                                 ModuleGUI *gui);
 static void             update_graph_model_props(GwyGraphModel *gmodel,
                                                  ModuleArgs *args);
+static gdouble          polynomial              (gdouble *param,
+                                                 gint degree,
+                                                 gdouble x);
 static void             sanitise_params         (ModuleArgs *args);
 
 static GwyModuleInfo module_info = {
@@ -153,11 +164,14 @@ define_module_params(void)
     gwy_param_def_add_int(paramdef, PARAM_XPOS, "xpos", NULL, -1, G_MAXINT, -1);
     gwy_param_def_add_int(paramdef, PARAM_YPOS, "ypos", NULL, -1, G_MAXINT, -1);
     /* Keep calling it order for compatibility. */
-    gwy_param_def_add_int(paramdef, PARAM_DEGREE, "order", _("_Degree"), 0, 5, 2);
+    gwy_param_def_add_int(paramdef, PARAM_DEGREE, "order", _("_Degree"), 0, MAXFITPARAMS-1, 2);
     gwy_param_def_add_double(paramdef, PARAM_RANGE_FROM, "from", _("_From"), 0.0, 1.0, 0.0);
     gwy_param_def_add_double(paramdef, PARAM_RANGE_TO, "to", _("_To"), 0.0, 1.0, 1.0);
     gwy_param_def_add_lawn_segment(paramdef, PARAM_SEGMENT, "segment", NULL);
     gwy_param_def_add_boolean(paramdef, PARAM_ENABLE_SEGMENT, "enable_segment", NULL, FALSE);
+    /* XXX: ’polynome’ is not a word. */
+    gwy_param_def_add_boolean(paramdef, PARAM_MEAN_CURVE, "mean_curve", _("Use _mean polynome"), FALSE);
+    gwy_param_def_add_boolean(paramdef, PARAM_APPLY_TO_ALL, "split", _("Apply to _all segments"), FALSE);
 
     return paramdef;
 }
@@ -182,6 +196,7 @@ polylevel(GwyContainer *data, GwyRunType runtype)
     args.lawn = lawn;
     args.nsegments = gwy_lawn_get_n_segments(lawn);
     args.params = gwy_params_new_from_settings(define_module_params());
+    args.meanfitparams = g_new(gdouble, MAXFITPARAMS);
     sanitise_params(&args);
 
     args.field = gwy_data_field_new(gwy_lawn_get_xres(lawn), gwy_lawn_get_yres(lawn),
@@ -199,12 +214,13 @@ polylevel(GwyContainer *data, GwyRunType runtype)
     if (outcome != GWY_DIALOG_HAVE_RESULT)
         execute(&args);
 
-    /* TODO: Add log, maybe more. */
+    gwy_app_curve_map_log_add(data, -1, id, "cmap::cmap_polylevel", NULL);
 
 end:
     GWY_OBJECT_UNREF(args.result);
     g_object_unref(args.field);
     g_object_unref(args.params);
+    g_free(args.meanfitparams);
 }
 
 static GwyDialogOutcome
@@ -222,6 +238,8 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     gwy_clear(&gui, 1);
     gui.args = args;
     gui.data = gwy_container_new();
+    gui.meanfitdone = FALSE;
+ 
     field = gwy_container_get_object(data, gwy_app_get_lawn_preview_key_for_id(id));
     gwy_container_set_object(gui.data, gwy_app_get_data_key_for_id(0), field);
     if (gwy_container_gis_string(data, gwy_app_get_lawn_palette_key_for_id(id), &gradient))
@@ -262,18 +280,15 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     gwy_graph_model_add_curve(gui.gmodel, gcmodel);
     g_object_unref(gcmodel);
 
-
-
     graph = gwy_graph_new(gui.gmodel);
     area = gwy_graph_get_area(GWY_GRAPH(graph));
     gwy_graph_enable_user_input(GWY_GRAPH(graph), FALSE);
     gwy_graph_area_set_status(GWY_GRAPH_AREA(area), GWY_GRAPH_STATUS_XSEL);
-    gwy_graph_area_set_selection_editable(GWY_GRAPH_AREA(area), FALSE);
+    gwy_graph_area_set_selection_editable(GWY_GRAPH_AREA(area), TRUE);
     gui.graph_selection = gwy_graph_area_get_selection(GWY_GRAPH_AREA(area), GWY_GRAPH_STATUS_XSEL);
     gtk_widget_set_size_request(graph, PREVIEW_SIZE, PREVIEW_SIZE);
     gtk_box_pack_start(GTK_BOX(hbox), graph, TRUE, TRUE, 0);
 
-
     hbox = gwy_hbox_new(20);
     gwy_dialog_add_content(GWY_DIALOG(gui.dialog), hbox, TRUE, TRUE, 4);
 
@@ -291,25 +306,41 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
         gwy_param_table_append_lawn_segment(table, PARAM_SEGMENT, args->lawn);
         gwy_param_table_add_enabler(table, PARAM_ENABLE_SEGMENT, PARAM_SEGMENT);
     }
+    gwy_dialog_add_param_table(dialog, table);
+    gtk_box_pack_start(GTK_BOX(hbox), gwy_param_table_widget(table), FALSE, FALSE, 0);
 
+    table = gui.table_output = gwy_param_table_new(args->params);
+    gwy_param_table_append_checkbox(table, PARAM_MEAN_CURVE);
+    if (args->nsegments) {
+        gwy_param_table_append_checkbox(table, PARAM_APPLY_TO_ALL);
+    }
     gwy_dialog_add_param_table(dialog, table);
     gtk_box_pack_start(GTK_BOX(hbox), gwy_param_table_widget(table), FALSE, FALSE, 0);
 
     g_signal_connect_swapped(gui.table, "param-changed", G_CALLBACK(param_changed), &gui);
+    g_signal_connect_swapped(gui.table_output, "param-changed", G_CALLBACK(param_changed), &gui);
     g_signal_connect_swapped(gui.selection, "changed", G_CALLBACK(point_selection_changed), &gui);
+    g_signal_connect(gui.graph_selection, "changed", G_CALLBACK(graph_selected), &gui);    
     gwy_dialog_set_preview_func(dialog, GWY_PREVIEW_IMMEDIATE, preview, &gui, NULL);
 
     outcome = gwy_dialog_run(dialog);
 
     g_object_unref(gui.gmodel);
     g_object_unref(gui.data);
-
     return outcome;
 }
 
 static void
-param_changed(ModuleGUI *gui, G_GNUC_UNUSED gint id)
+param_changed(ModuleGUI *gui, gint id)
 {
+    ModuleArgs *args = gui->args;
+    GwyParams *params = args->params;
+
+    if (id < 0 || id == PARAM_ENABLE_SEGMENT) {
+        gwy_param_table_set_sensitive(gui->table_output, PARAM_APPLY_TO_ALL,
+                                      gwy_params_get_boolean(params, PARAM_ENABLE_SEGMENT));
+    }
+
     gwy_dialog_invalidate(GWY_DIALOG(gui->dialog));
 }
 
@@ -344,28 +375,95 @@ point_selection_changed(ModuleGUI *gui, gint id, GwySelection *selection)
     gwy_param_table_param_changed(gui->table, PARAM_YPOS);
 }
 
+static void
+graph_selected(GwySelection* selection, gint i, ModuleGUI *gui)
+{
+    gdouble range[2];
+    gdouble xfrom, xto, cxfrom, cxto;
+    gboolean have_range = TRUE;
+    GwyGraphCurveModel *gc;
+    
+    g_return_if_fail(i <= 0);
+    
+    if (gwy_selection_get_data(selection, NULL) <= 0)
+        have_range = FALSE;
+    else {
+        gwy_selection_get_object(selection, 0, range);
+        if (range[0] == range[1])
+            have_range = FALSE;
+    }
+    if (have_range) {
+        gc = gwy_graph_model_get_curve(gui->gmodel, 0);
+        gwy_graph_curve_model_get_x_range(gc, &cxfrom, &cxto);
+
+        xfrom = MIN(range[0], range[1]);
+        xto = MAX(range[0], range[1]);
+
+        xfrom = MAX(xfrom, cxfrom);
+        xto = MIN(xto, cxto);
+    
+        gwy_param_table_set_double(gui->table, PARAM_RANGE_FROM, (xfrom - cxfrom)/(cxto - cxfrom));
+        gwy_param_table_set_double(gui->table, PARAM_RANGE_TO, (xto - cxfrom)/(cxto - cxfrom));
+    }
+}
+
 static void
 preview(gpointer user_data)
 {
     ModuleGUI *gui = (ModuleGUI*)user_data;
     ModuleArgs *args = gui->args;
+    GwyLawn *lawn = args->lawn;
     GwyParams *params = args->params;
     gint col = gwy_params_get_int(params, PARAM_XPOS);
     gint row = gwy_params_get_int(params, PARAM_YPOS);
     gdouble from = gwy_params_get_double(params, PARAM_RANGE_FROM);
     gdouble to = gwy_params_get_double(params, PARAM_RANGE_TO);
+    gboolean mc = gwy_params_get_boolean(params, PARAM_MEAN_CURVE);
+    gint degree = gwy_params_get_int(params, PARAM_DEGREE);
     gdouble sel[2];
     GwyGraphCurveModel *gc;
     gdouble xfrom, xto;
     gdouble *xfit, *yfit, *fitparams, x;
-    gint i, nfit = 100;
+    gint i, k, scol, srow, nfit = 100;
+    gint xres = gwy_lawn_get_xres(lawn), yres = gwy_lawn_get_yres(lawn);
 
     fitparams = g_new(gdouble, 6);
-
     gc = gwy_graph_model_get_curve(gui->gmodel, 0);
-    extract_one_curve(args->lawn, gc, col, row, params);
-    convert_one_curve(gc, params, fitparams,  args->nsegments,
-                      gwy_lawn_get_segments(args->lawn, col, row, NULL));
+
+    if (mc) {
+        if (!gui->meanfitdone) {
+            for (i = 0; i < MAXFITPARAMS; i++)
+                args->meanfitparams[i] = 0;
+
+            for (k = 0; k < xres*yres; k++) {
+                scol = k % xres;
+                srow = k/xres;
+
+                extract_one_curve(lawn, gc, scol, srow, params);
+
+                convert_one_curve(gc, params, fitparams, args->nsegments,
+                                  gwy_lawn_get_segments(lawn, scol, srow, NULL));
+
+                for (i = 0; i < MAXFITPARAMS; i++)
+                    args->meanfitparams[i] += fitparams[i];
+            }
+            for (i = 0; i < MAXFITPARAMS; i++) {
+                args->meanfitparams[i] /= (xres*yres);
+                fitparams[i] = args->meanfitparams[i];
+            }
+            gui->meanfitdone = TRUE;
+        }
+
+        for (i = 0; i < MAXFITPARAMS; i++)
+            fitparams[i] = args->meanfitparams[i];
+
+        extract_one_curve(lawn, gc, col, row, params);
+    }
+    else {
+        extract_one_curve(lawn, gc, col, row, params);
+        convert_one_curve(gc, params, fitparams, args->nsegments,
+                          gwy_lawn_get_segments(lawn, col, row, NULL));
+    }
     update_graph_model_props(gui->gmodel, args);
 
     gwy_graph_curve_model_get_x_range(gc, &xfrom, &xto);
@@ -378,8 +476,7 @@ preview(gpointer user_data)
     yfit = g_new(gdouble, nfit);
     for (i = 0; i < nfit; i++) {
         x = xfit[i];
-        yfit[i] = fitparams[0] + x*(fitparams[1] + x*(fitparams[2] + x*(fitparams[3]
-                                                                        + x*(fitparams[4] + x*fitparams[5]))));
+        yfit[i] = polynomial(fitparams, degree, x);
     }
     gc = gwy_graph_model_get_curve(gui->gmodel, 1);
     gwy_graph_curve_model_set_data(gc, xfit, yfit, nfit);
@@ -400,7 +497,9 @@ execute(ModuleArgs *args)
     gdouble to = gwy_params_get_double(params, PARAM_RANGE_TO);
     gint degree = gwy_params_get_int(params, PARAM_DEGREE);
     gboolean segment_enabled = args->nsegments ? gwy_params_get_boolean(params, PARAM_ENABLE_SEGMENT) : FALSE;
+    gboolean apply_to_all = gwy_params_get_boolean(params, PARAM_APPLY_TO_ALL);
     gint segment = segment_enabled ? gwy_params_get_int(params, PARAM_SEGMENT) : -1;
+    gboolean mc = gwy_params_get_boolean(params, PARAM_MEAN_CURVE);
 
     GwyLawn *lawn = args->lawn;
     gint xres = gwy_lawn_get_xres(lawn), yres = gwy_lawn_get_yres(lawn);
@@ -409,7 +508,7 @@ execute(ModuleArgs *args)
 
     const gdouble *cdx, *cdy;
     gdouble *ncdy;
-    gint ndata, k, col, row;
+    gint ndata, i, k, col, row;
 
     gint nsegments = gwy_lawn_get_n_segments(lawn);
     gint *segments = g_new(gint, 2*nsegments);
@@ -425,7 +524,14 @@ execute(ModuleArgs *args)
         g_array_set_size(newcdata, 2*ndata);
         ncdy = &g_array_index(newcdata, gdouble, ndata);
 
-        do_polylevel(cdx, cdy, ncdy, ndata, segments, segment, segment_enabled, from, to, degree, TRUE, NULL);
+        if (mc) {
+            for (i = 0; i < ndata; i++) {
+               ncdy[i] = cdy[i] - polynomial(args->meanfitparams, degree, cdx[i]);
+            }
+        } else {
+            do_polylevel(cdx, cdy, ncdy, ndata, segments, segment, segment_enabled, apply_to_all, 
+                         from, to, degree, TRUE, NULL);
+        }
 
         gwy_lawn_set_curve_data(lawn, col, row, ordinate, ncdy);
     }
@@ -446,6 +552,21 @@ extract_one_curve(GwyLawn *lawn, GwyGraphCurveModel *gcmodel,
     gwy_graph_curve_model_set_data(gcmodel, xdata, ydata, ndata);
 }
 
+static gdouble
+polynomial(gdouble *param, gint degree, gdouble x)
+{
+    gdouble val;
+    gint i;
+    val = param[0];
+    
+    val = param[degree];
+    for (i = degree; i >= 1; i--) {
+        val = val*x + param[i-1];
+    }
+    return val;
+}
+
+
 //subtract means that the data will be changed, which is good for final operation
 //fitparams are used to get the fit results and plot it with original data, which is good for GUI
 // FIXME: Almost duplicate with cmap_lockin.c
@@ -453,14 +574,14 @@ static void
 do_polylevel(const gdouble *xdata, const gdouble *ydata,
              gdouble *nydata, gint ndata,
              const gint *segments,
-             gint segment, gboolean segment_enabled,
+             gint segment, gboolean segment_enabled, gboolean apply_to_all,
              gdouble from, gdouble to,
              gint degree, gboolean subtract, gdouble *fitparams)
 {
     gint i, j, n;
     gdouble startval, endval, xmin, xmax, ymin, ymax;
     gdouble *xf, *yf;
-    gdouble param[6], x;
+    gdouble param[MAXFITPARAMS], x;
     gint segment_from, segment_to;
 
     /* FIXME: This function must be able to fail gracefully and report it to the caller – the fitting can definitely
@@ -513,12 +634,19 @@ do_polylevel(const gdouble *xdata, const gdouble *ydata,
     if (subtract && nydata) {
         for (i = 0; i < ndata; i++) {
             x = xdata[i];
-            nydata[i] = ydata[i] - (param[0] + x*(param[1] + x*(param[2] + x*(param[3] + x*(param[4] + x*param[5])))));
+            if (!segment_enabled || apply_to_all) 
+                nydata[i] = ydata[i] - polynomial(param, degree, x);
+            else {
+                if (i >= segment_from && i < segment_to)
+                    nydata[i] = ydata[i] - polynomial(param, degree, x);
+                else
+                    nydata[i] = ydata[i];
+            }
         }
     }
 
     if (fitparams)
-        gwy_assign(fitparams, param, 6);
+        gwy_assign(fitparams, param, MAXFITPARAMS);
 
     g_free(xf);
     g_free(yf);
@@ -532,13 +660,15 @@ convert_one_curve(GwyGraphCurveModel *gcmodel, GwyParams *params, gdouble *fitpa
     gdouble to = gwy_params_get_double(params, PARAM_RANGE_TO);
     gint degree = gwy_params_get_int(params, PARAM_DEGREE);
     gboolean segment_enabled = nsegments ? gwy_params_get_boolean(params, PARAM_ENABLE_SEGMENT) : FALSE;
+    gboolean apply_to_all = gwy_params_get_boolean(params, PARAM_APPLY_TO_ALL);
     gint segment = segment_enabled ? gwy_params_get_int(params, PARAM_SEGMENT) : -1;
 
     const gdouble *xdata = gwy_graph_curve_model_get_xdata(gcmodel);
     const gdouble *ydata = gwy_graph_curve_model_get_ydata(gcmodel);
     gint ndata = gwy_graph_curve_model_get_ndata(gcmodel);
 
-    do_polylevel(xdata, ydata, NULL, ndata, segments, segment, segment_enabled, from, to, degree, FALSE, fitparams);
+    do_polylevel(xdata, ydata, NULL, ndata, segments, segment, segment_enabled, apply_to_all,
+                 from, to, degree, FALSE, fitparams);
 }
 
 static void
