@@ -1,5 +1,5 @@
 /*
- *  $Id: cmap_simplemech.c 26949 2024-12-03 10:42:17Z yeti-dn $
+ *  $Id: cmap_simplemech.c 28321 2025-08-02 10:49:18Z klapetek $
  *  Copyright (C) 2021 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -738,7 +738,7 @@ evaluate_curve(const gdouble *xdata, const gdouble *ydata,
     }
 
     //find dmt fit limits as points where we reach the points on retract curve when going from peak value
-    xupper = xpeak;     //safe inits
+    xupper = rfrom;//xpeak;     //safe inits
     xlower = xadhesion;
     iupper = 0;
     ilower = iadhesion;
@@ -746,10 +746,10 @@ evaluate_curve(const gdouble *xdata, const gdouble *ydata,
     lowerval = adhesion + fit_lower*(peak-adhesion);
 
     for (i = (nrdata - 1); i > 1; i--) {
-        if (yrdata[i] >= upperval && yrdata[i + 1] < upperval) {
-            xupper = xrdata[i];
-            iupper = i;
-        }
+//        if (yrdata[i] >= upperval && yrdata[i + 1] < upperval) {
+//            xupper = xrdata[i];
+//            iupper = i;
+//        }
         if (yrdata[i] >= lowerval && yrdata[i+1] < lowerval) {
             xlower = xrdata[i];
             ilower = i;
