@@ -1,5 +1,5 @@
 /*
- *  $Id: cmap_sinebg.c 26949 2024-12-03 10:42:17Z yeti-dn $
+ *  $Id: cmap_sinebg.c 28888 2025-11-20 11:37:15Z klapetek $
  *  Copyright (C) 2021 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -112,7 +112,9 @@ static gdouble          func_sine               (gdouble x,
                                                  const gdouble *param,
                                                  G_GNUC_UNUSED gpointer user_data,
                                                  gboolean *fres);
-
+static void             graph_selected          (GwySelection* selection,
+                                                 gint i,
+                                                 ModuleGUI *gui);
 static void             update_graph_model_props(GwyGraphModel *gmodel,
                                                  ModuleArgs *args);
 static void             sanitise_params         (ModuleArgs *args);
@@ -203,7 +205,7 @@ sinebg(GwyContainer *data, GwyRunType runtype)
         execute(&args, gwy_app_find_window_for_curve_map(data, id));
     }
 
-    /* TODO: Add log, maybe more. */
+    gwy_app_curve_map_log_add(data, -1, id, "cmap::cmap_sinebg", NULL);
 
 end:
     GWY_OBJECT_UNREF(args.result);
@@ -272,7 +274,7 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
     area = gwy_graph_get_area(GWY_GRAPH(graph));
     gwy_graph_enable_user_input(GWY_GRAPH(graph), FALSE);
     gwy_graph_area_set_status(GWY_GRAPH_AREA(area), GWY_GRAPH_STATUS_XSEL);
-    gwy_graph_area_set_selection_editable(GWY_GRAPH_AREA(area), FALSE);
+    gwy_graph_area_set_selection_editable(GWY_GRAPH_AREA(area), TRUE);
     gui.graph_selection = gwy_graph_area_get_selection(GWY_GRAPH_AREA(area), GWY_GRAPH_STATUS_XSEL);
     gtk_widget_set_size_request(graph, PREVIEW_SIZE, PREVIEW_SIZE);
     gtk_box_pack_start(GTK_BOX(hbox), graph, TRUE, TRUE, 0);
@@ -300,6 +302,7 @@ run_gui(ModuleArgs *args, GwyContainer *data, gint id)
 
     g_signal_connect_swapped(gui.table, "param-changed", G_CALLBACK(param_changed), &gui);
     g_signal_connect_swapped(gui.selection, "changed", G_CALLBACK(point_selection_changed), &gui);
+    g_signal_connect(gui.graph_selection, "changed", G_CALLBACK(graph_selected), &gui);
     gwy_dialog_set_preview_func(dialog, GWY_PREVIEW_IMMEDIATE, preview, &gui, NULL);
 
     outcome = gwy_dialog_run(dialog);
@@ -347,6 +350,38 @@ point_selection_changed(ModuleGUI *gui, gint id, GwySelection *selection)
     gwy_param_table_param_changed(gui->table, PARAM_YPOS);
 }
 
+static void
+graph_selected(GwySelection* selection, gint i, ModuleGUI *gui)
+{   
+    gdouble range[2];
+    gdouble xfrom, xto, cxfrom, cxto;
+    gboolean have_range = TRUE;
+    GwyGraphCurveModel *gc;
+
+    g_return_if_fail(i <= 0);
+    
+    if (gwy_selection_get_data(selection, NULL) <= 0)
+        have_range = FALSE;
+    else {
+        gwy_selection_get_object(selection, 0, range);
+        if (range[0] == range[1])
+            have_range = FALSE;
+    }
+    if (have_range) {
+        gc = gwy_graph_model_get_curve(gui->gmodel, 0);
+        gwy_graph_curve_model_get_x_range(gc, &cxfrom, &cxto);
+
+        xfrom = MIN(range[0], range[1]); 
+        xto = MAX(range[0], range[1]);
+
+        xfrom = MAX(xfrom, cxfrom);
+        xto = MIN(xto, cxto);
+    
+        gwy_param_table_set_double(gui->table, PARAM_RANGE_FROM, (xfrom - cxfrom)/(cxto - cxfrom));
+        gwy_param_table_set_double(gui->table, PARAM_RANGE_TO, (xto - cxfrom)/(cxto - cxfrom));
+    }
+}
+
 static void
 preview(gpointer user_data)
 {
