@@ -60,6 +60,7 @@ GwyModuleInfo* _gwy_module_query__iso28600(void);
 GwyModuleInfo* _gwy_module_query__jeol(void);
 GwyModuleInfo* _gwy_module_query__jeoltem(void);
 GwyModuleInfo* _gwy_module_query__jspmfile(void);
+GwyModuleInfo* _gwy_module_query__lafmdmap(void);
 GwyModuleInfo* _gwy_module_query__leica(void);
 GwyModuleInfo* _gwy_module_query__lextfile(void);
 GwyModuleInfo* _gwy_module_query__magellan(void);
@@ -97,6 +98,7 @@ GwyModuleInfo* _gwy_module_query__pltfile(void);
 GwyModuleInfo* _gwy_module_query__princetonspe(void);
 GwyModuleInfo* _gwy_module_query__psia(void);
 GwyModuleInfo* _gwy_module_query__pt3file(void);
+GwyModuleInfo* _gwy_module_query__qnami(void);
 GwyModuleInfo* _gwy_module_query__quazarnpic(void);
 GwyModuleInfo* _gwy_module_query__quesant(void);
 GwyModuleInfo* _gwy_module_query__rawfile(void);
@@ -189,6 +191,7 @@ static const GwyModuleRecord modules[] = {
   { _gwy_module_query__jeol, "jeol", },
   { _gwy_module_query__jeoltem, "jeoltem", },
   { _gwy_module_query__jspmfile, "jspmfile", },
+  { _gwy_module_query__lafmdmap, "lafmdmap", },
   { _gwy_module_query__leica, "leica", },
   { _gwy_module_query__lextfile, "lextfile", },
   { _gwy_module_query__magellan, "magellan", },
@@ -226,6 +229,7 @@ static const GwyModuleRecord modules[] = {
   { _gwy_module_query__princetonspe, "princetonspe", },
   { _gwy_module_query__psia, "psia", },
   { _gwy_module_query__pt3file, "pt3file", },
+  { _gwy_module_query__qnami, "qnami", },
   { _gwy_module_query__quazarnpic, "quazarnpic", },
   { _gwy_module_query__quesant, "quesant", },
   { _gwy_module_query__rawfile, "rawfile", },
