@@ -1,5 +1,5 @@
 /*
- *  $Id: gwytiff.c 28092 2025-06-11 14:23:18Z yeti-dn $
+ *  $Id: gwytiff.c 28461 2025-08-27 14:57:13Z yeti-dn $
  *  Copyright (C) 2007-2025 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -648,8 +648,13 @@ gwy_tiff_get_float_entry(const GwyTIFF *tiff,
     else if (type == GWY_TIFF_DOUBLE)
         *retval = tiff->get_gdouble(&p);
     else if (type == GWY_TIFF_RATIONAL) {
-        gint numerator = tiff->get_gint32(&p);
-        gint denominator = tiff->get_gint32(&p);
+        guint numerator = tiff->get_guint32(&p);
+        guint denominator = tiff->get_guint32(&p);
+        *retval = (gdouble)numerator/denominator;
+    }
+    else if (type == GWY_TIFF_SRATIONAL) {
+        guint numerator = tiff->get_gint32(&p);
+        guint denominator = tiff->get_gint32(&p);
         *retval = (gdouble)numerator/denominator;
     }
     else
