@@ -1,5 +1,5 @@
 /*
- *  $Id: epflfile.c 26873 2024-11-19 12:52:14Z yeti-dn $
+ *  $Id: epflfile.c 28533 2025-09-10 14:07:50Z yeti-dn $
  *  Copyright (C) 2020-2024 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -123,6 +123,11 @@ epfl_detect(const GwyFileDetectInfo *fileinfo,
     if ((file_id = gwyhdf5_quick_check(fileinfo, only_name)) < 0)
         return 0;
 
+    if (!H5Lexists(file_id, "measurements", H5P_DEFAULT)) {
+        H5Fclose(file_id);
+        return 0;
+    }
+
     /* The files do not have any of the typical top-level attributes we then use to recognise the data formats.
      * Look for some groups. They should be always present, even if they are empty. */
     if ((group1_id = H5Gopen(file_id, "measurements", H5P_DEFAULT)) > 0) {
