@@ -1,5 +1,5 @@
 /*
- *  $Id: ergofile.c 26873 2024-11-19 12:52:14Z yeti-dn $
+ *  $Id: ergofile.c 28724 2025-10-23 13:43:19Z yeti-dn $
  *  Copyright (C) 2020-2024 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -54,6 +54,7 @@ typedef struct {
     GArray *channels;
     gint nframes;
     const gchar *filename;
+    gint version[3];
 } ErgoFile;
 
 static gint          ergo_detect       (const GwyFileDetectInfo *fileinfo,
@@ -68,6 +69,7 @@ static GwyContainer* ergo_read_channels(hid_t file_id,
                                         GwyHDF5File *ghfile,
                                         GError **error);
 static GwyDataField* ergo_read_field   (hid_t file_id,
+                                        gint version_major,
                                         guint r,
                                         ErgoChannel *c,
                                         gint frameid,
@@ -98,10 +100,12 @@ ergo_detect(const GwyFileDetectInfo *fileinfo,
     if ((file_id = gwyhdf5_quick_check(fileinfo, only_name)) < 0)
         return 0;
 
-    if (gwyhdf5_get_str_attr(file_id, ".", "ARFormat", &format, NULL)) {
-        if (gwyhdf5_get_ints_attr(file_id, ".", "ARVersion", 1, &dim, version, NULL))
-            score = 100;
-        H5free_memory(format);
+    if (H5Aexists(file_id, "ARFormat") && H5Aexists(file_id, "ARVersion")) {
+        if (gwyhdf5_get_str_attr(file_id, ".", "ARFormat", &format, NULL)) {
+            if (gwyhdf5_get_ints_attr(file_id, ".", "ARVersion", 1, &dim, version, NULL))
+                score = 100;
+            H5free_memory(format);
+        }
     }
 
     H5Fclose(file_id);
@@ -120,6 +124,8 @@ ergo_load(const gchar *filename,
     hid_t file_id;
     G_GNUC_UNUSED herr_t status;
     H5O_info_t infobuf;
+    gint ver_dim = G_N_ELEMENTS(efile.version);
+    const gchar *prefix;
     guint i;
 
     if ((file_id = H5Fopen(filename, H5F_ACC_RDONLY, H5P_DEFAULT)) < 0) {
@@ -131,15 +137,27 @@ ergo_load(const gchar *filename,
     if (!gwyhdf5_check_status(status, file_id, NULL, "H5Oget_info", error))
         return NULL;
 
+    gwy_clear(&efile, 1);
+    if (!gwyhdf5_get_ints_attr(file_id, ".", "ARVersion", 1, &ver_dim, efile.version, NULL))
+        g_warning("Cannot determine AR Version");
+    else if (efile.version[0] != 1 && efile.version[0] != 2)
+        g_warning("ARVersion = %d.%d.%d is not implemented", efile.version[0], efile.version[1], efile.version[2]);
+
     gwyhdf5_init(&ghfile);
     ghfile.impl = &efile;
     ghfile.attr_handler = ergo_attr_handler;
     gwyhdf5_alloc_lists(&ghfile, 1);
-    ghfile.lists[0].idprefix = "/DataSet/Resolution ";
+    if (efile.version[0] >= 2) {
+        gwy_debug("assuming version >= 2");
+        ghfile.lists[0].idprefix = "/Image/DataSet/Resolution ";
+    }
+    else {
+        gwy_debug("assuming version <= 1");
+        ghfile.lists[0].idprefix = "/DataSet/Resolution ";
+    }
     ghfile.lists[0].idwhat = H5O_TYPE_GROUP;
     g_array_append_val(ghfile.addr, infobuf.addr);
 
-    gwy_clear(&efile, 1);
     efile.filename = filename;
     efile.channels = g_array_new(FALSE, FALSE, sizeof(ErgoChannel));
 
@@ -158,7 +176,12 @@ ergo_load(const gchar *filename,
     g_object_unref(ghfile.meta);
     ghfile.meta = meta;
 
-    if (gwyhdf5_get_int_attr(file_id, "DataSetInfo", "NumFrames", &efile.nframes, error)) {
+    if (efile.version[0] >= 2)
+        prefix = "Image/DataSetInfo";
+    else
+        prefix = "DataSetInfo";
+
+    if (gwyhdf5_get_int_attr(file_id, prefix, "NumFrames", &efile.nframes, error)) {
         gwy_debug("nframes %d", efile.nframes);
         container = ergo_read_channels(file_id, &ghfile, error);
     }
@@ -191,14 +214,20 @@ ergo_read_channels(hid_t file_id, GwyHDF5File *ghfile, GError **error)
     gint expected2[2] = { 2, 2 }, yxres[2];
     gchar *s, *s2[2];
     gint frameid, id = 0;
+    const gchar *prefix;
     guint i, ri, r;
 
+    if (efile->version[0] >= 2)
+        prefix = "Image/DataSetInfo";
+    else
+        prefix = "DataSetInfo";
+
     for (ri = 0; ri < resolutions->len; ri++) {
         r = g_array_index(resolutions, guint, ri);
         for (i = 0; i < channels->len; i++) {
             ErgoChannel *c = &g_array_index(channels, ErgoChannel, i);
 
-            g_string_printf(str, "DataSetInfo/Global/Channels/%s/ImageDims", c->name);
+            g_string_printf(str, "%s/Global/Channels/%s/ImageDims", prefix, c->name);
 
             if (!gwyhdf5_get_str_attr(file_id, str->str, "DataUnits", &s, error))
                 goto fail;
@@ -227,7 +256,7 @@ ergo_read_channels(hid_t file_id, GwyHDF5File *ghfile, GError **error)
             gwy_debug("resid %u res %dx%d", r, yxres[1], yxres[0]);
 
             for (frameid = 0; frameid < efile->nframes; frameid++) {
-                if (!(dfield = ergo_read_field(file_id, r, c, frameid, yxres, str, &err))) {
+                if (!(dfield = ergo_read_field(file_id, efile->version[0], r, c, frameid, yxres, str, &err))) {
                     /* Sometimes this happens? Perhaps interrupted measurement? */
                     g_warning("Cannot read %s: %s", str->str, err->message);
                     g_clear_error(&err);
@@ -262,6 +291,7 @@ fail:
 
 static GwyDataField*
 ergo_read_field(hid_t file_id,
+                gint version_major,
                 guint r, ErgoChannel *c, gint frameid, const gint *yxres,
                 GString *str, GError **error)
 {
@@ -269,8 +299,14 @@ ergo_read_field(hid_t file_id,
     hid_t dataset;
     gdouble q, xreal, yreal, xoff, yoff;
     herr_t status;
+    const gchar *prefix;
 
-    g_string_printf(str, "DataSet/Resolution %u/Frame %d/%s/Image", r, frameid, c->name);
+    if (version_major >= 2)
+        prefix = "Image/DataSet";
+    else
+        prefix = "DataSet";
+
+    g_string_printf(str, "%s/Resolution %u/Frame %d/%s/Image", prefix, r, frameid, c->name);
     if ((dataset = gwyhdf5_open_and_check_dataset(file_id, str->str, 2, (gint*)yxres, error)) < 0)
         return NULL;
 
@@ -326,18 +362,24 @@ ergo_attr_handler(GwyHDF5File *ghfile, hid_t loc_id, const char *attr_name)
     gboolean is_vlenstr = FALSE;
     gint nitems, i;
     herr_t status;
+    const gchar *path;
 
-    if (!gwy_strequal(ghfile->path->str, "/DataSetInfo/ChannelNames"))
+    if (efile->version[0] >= 2)
+        path = "/Image/DataSetInfo/ChannelNames";
+    else
+        path = "/DataSetInfo/ChannelNames";
+
+    if (!gwy_strequal(ghfile->path->str, path))
         return;
 
-    gwy_debug("handling /DataSetInfo/ChannelNames");
+    gwy_debug("handling %s", path);
     attr = H5Aopen(loc_id, attr_name, H5P_DEFAULT);
     attr_type = H5Aget_type(attr);
     space = H5Aget_space(attr);
     nitems = H5Sget_simple_extent_npoints(space);
     type_class = H5Tget_class(attr_type);
 
-    if (gwy_strequal(ghfile->path->str, "/DataSetInfo/ChannelNames")) {
+    if (gwy_strequal(ghfile->path->str, path)) {
         if (type_class == H5T_STRING) {
             is_vlenstr = H5Tis_variable_str(attr_type);
         }
