@@ -1,5 +1,5 @@
 /*
- *  $Id: gwyhdf5.c 26518 2024-08-15 18:03:35Z yeti-dn $
+ *  $Id: gwyhdf5.c 28998 2025-12-15 13:20:20Z yeti-dn $
  *  Copyright (C) 2020-2024 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -314,8 +314,10 @@ process_fixed_string_attribute(hid_t attr, hid_t attr_type, gint nitems,
 
     if (nitems == 1) {
         gwy_debug("fixed-size string");
-        g_string_set_size(outbuf, nitems);
         str_type = gwyhdf5_make_string_type_for_attr(attr_type);
+        /* NB: attr_type's size is only the string content size, not taking into account the C representation with
+         * padding nul. */
+        g_string_set_size(outbuf, H5Tget_size(str_type));
         if ((status = H5Aread(attr, str_type, outbuf->str)) >= 0) {
             g_strstrip(outbuf->str);
             outbuf->len = strlen(outbuf->str);
@@ -842,16 +844,16 @@ gwyhdf5_get_strs_attr(hid_t file_id,
     }
     is_vlenstr = H5Tis_variable_str(attr_type);
     gwy_debug("attr %d is%s vlen string", (gint)attr, is_vlenstr ? "" : " not");
-    if (!is_vlenstr) {
-        H5Tclose(attr_type);
-        H5Aclose(attr);
-        /* XXX: Be more specific. */
-        err_UNSUPPORTED(error, attr_name);
-        return FALSE;
-    }
 
     str_type = gwyhdf5_make_string_type_for_attr(attr_type);
-    status = H5Aread(attr, str_type, v);
+    if (is_vlenstr) {
+        status = H5Aread(attr, str_type, v);
+    }
+    else {
+        gchar *s = g_new(gchar, H5Tget_size(attr_type)+1);
+        status = H5Aread(attr, str_type, s);
+        *v = s;
+    }
     H5Tclose(attr_type);
     H5Tclose(str_type);
     H5Aclose(attr);
@@ -921,6 +923,8 @@ gwyhdf5_get_simple_space_dims(hid_t space, gint expected_ndims, gint *dims,
     return TRUE;
 }
 
+/* NB: Pass 0 as expected_ndims and NULL for dims for SCALAR datasets (scalars are not one-element arrays; they are
+ * zero-dimension point-like objects in HDF5). */
 hid_t
 gwyhdf5_open_and_check_dataset(hid_t file_id, const gchar *name,
                                gint expected_ndims, gint *dims,
@@ -949,6 +953,59 @@ gwyhdf5_open_and_check_dataset(hid_t file_id, const gchar *name,
     return dataset;
 }
 
+/* Read string stored as a dataset. Fortunately, only GXSM4 does things like that. So we only support the moderately
+ * insane case of strings as arrays of one-character little-strings, but nothing more complicated. */
+gchar*
+gwyhdf5_read_string_dataset(hid_t loc_id, const gchar *name, GError **error)
+{
+    hid_t dataset, dataset_type, str_type;
+    H5T_class_t type_class;
+    gint status, len = -1;
+    size_t type_size;
+    gchar *s = NULL;
+    gboolean is_vlenstr;
+
+    if ((dataset = gwyhdf5_open_and_check_dataset(loc_id, name, 1, &len, error)) < 0)
+        return NULL;
+
+    dataset_type = H5Dget_type(dataset);
+    type_class = H5Tget_class(dataset_type);
+    gwy_debug("len = %d, class = %d", len, type_class);
+    if (type_class != H5T_STRING)
+        goto end;
+
+    /* We only read string datasets which are simple strings formed by single characters. No
+     * array-of-variable-length-strings nonsense. */
+    is_vlenstr = H5Tis_variable_str(dataset_type);
+    if (is_vlenstr) {
+        err_UNSUPPORTED(error, "is_variable_str");
+        goto end;
+    }
+    type_size = H5Tget_size(dataset_type);
+    if (type_size != 1) {
+        err_UNSUPPORTED(error, "str_size");
+        goto end;
+    }
+
+    str_type = H5Tcopy(H5T_C_S1);
+    H5Tset_size(str_type, 1);
+    H5Tset_strpad(str_type, H5T_STR_NULLTERM);
+    H5Tset_cset(str_type, H5Tget_cset(dataset_type));
+    s = g_new0(gchar, len+1);
+    if ((status = H5Dread(dataset, str_type, H5S_ALL, H5S_ALL, H5P_DEFAULT, s)) < 0) {
+        err_HDF5(error, "H5Dread", status);
+        GWY_FREE(s);
+    }
+
+    H5Tclose(str_type);
+
+end:
+    H5Tclose(dataset_type);
+    H5Dclose(dataset);
+
+    return s;
+}
+
 /* Gather all the numbers in paths of the form prefix#NUM. */
 gboolean
 gwyhdf5_enumerate_indexed(const gchar *path, const gchar *prefix, GArray *array)
