@@ -1,5 +1,5 @@
 /*
- *  $Id: hdf5file.c 28040 2025-05-31 12:28:29Z yeti-dn $
+ *  $Id: hdf5file.c 29007 2025-12-15 16:09:18Z yeti-dn $
  *  Copyright (C) 2020-2024 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -79,7 +79,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Imports files based on Hierarchical Data Format (HDF), version 5."),
     "Yeti <yeti@gwyddion.net>",
-    "2.4",
+    "2.9",
     "David Nečas (Yeti) & Petr Klapetek",
     "2020",
 };
@@ -116,6 +116,10 @@ module_register(void)
     gwyhdf5_register_datxfile();
     gwyhdf5_register_epflfile();
     gwyhdf5_register_ergofile();
+#ifdef HAVE_JANSSON
+    gwyhdf5_register_fusionscope();
+#endif
+    gwyhdf5_register_gxsm4();
     gwyhdf5_register_nhffile();
     gwyhdf5_register_nsidfile();
     gwyhdf5_register_shilpsfile();
@@ -210,7 +214,7 @@ ghdf5_load(const gchar *filename,
     for (i = 0; i < ghfile.datasets->len; i++) {
         gchar *name = g_array_index(ghfile.datasets, gchar*, i);
         gint dims[2] = { -1, -1 };
-        GwyDataField *field;
+        GwyDataField *field, *mask;
 
         if ((dataset = gwyhdf5_open_and_check_dataset(file_id, name, 2, dims, NULL)) < 0)
             continue;
@@ -230,7 +234,11 @@ ghdf5_load(const gchar *filename,
         if (!container)
             container = gwy_container_new();
 
+        mask = gwy_app_channel_mask_of_nans(field, TRUE);
         gwy_container_pass_object(container, gwy_app_get_data_key_for_id(id), field);
+        if (mask)
+            gwy_container_pass_object(container, gwy_app_get_mask_key_for_id(id), mask);
+        gwy_file_channel_import_log_add(container, id, NULL, filename);
         id++;
     }
 
