@@ -1,5 +1,5 @@
 /*
- *  $Id: nhffile.c 26873 2024-11-19 12:52:14Z yeti-dn $
+ *  $Id: nhffile.c 29007 2025-12-15 16:09:18Z yeti-dn $
  *  Copyright (C) 2024 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -224,7 +224,9 @@ nhf_detect(const GwyFileDetectInfo *fileinfo,
     if ((file_id = gwyhdf5_quick_check(fileinfo, only_name)) < 0)
         return 0;
 
-    if (gwyhdf5_get_int_attr(file_id, ".", "nsf_file_version_major", &ver_major, NULL)
+    if (H5Aexists(file_id, "nsf_file_version_major")
+        && H5Aexists(file_id, "nsf_file_version_minor")
+        && gwyhdf5_get_int_attr(file_id, ".", "nsf_file_version_major", &ver_major, NULL)
         && gwyhdf5_get_int_attr(file_id, ".", "nsf_file_version_minor", &ver_minor, NULL)) {
         score = 70;
         if (gwyhdf5_get_str_attr(file_id, ".", "created", &created, NULL)) {
@@ -598,6 +600,7 @@ handle_curvelen_dataset(hid_t loc_id,
 
     cl[npts] = 0;
     for (i = 0; i < npts; i++) {
+        gwy_debug("[%d] %u", i, cl[i]);
         if (cl[i] > nhfile->maxlen) {
             err_INVALID(&nhfile->error, "dataset_block_size_max");
             status = NHF_FAIL;
@@ -642,13 +645,17 @@ handle_cmap_dataset(hid_t loc_id,
 
     status = NHF_NEUTRAL;
     gwy_clear(&cd, 1);
-    /* dataset_is_placeholder() reads "dataset_size" attribute which is the true total size (curve length sum), not
-     * the allocated size which is ndata */
+    /* dataset_is_placeholder() reads "dataset_size" attribute which used to be the true total size (curve length
+     * sum), but can apparently be either that or the allocated size (fullsize) */
     npts = nhfile->xres * nhfile->yres;
     ndata = npts*nhfile->maxlen;
+    gwy_debug("xres %d, yres %d, size %d, maxlen %d", nhfile->xres, nhfile->yres, size, nhfile->maxlen);
     if (size != nhfile->curve_lengths[npts]) {
-        gwy_debug("dataset has the wrong size %d (expecting %d values)", size, ndata);
-        return NHF_NEUTRAL;
+        gwy_debug("dataset has the wrong size %d (expecting true size %d values)", size, nhfile->curve_lengths[npts]);
+        if (size != ndata) {
+            gwy_debug("dataset has the wrong size also for maxlen (virtual %d values), giving up", ndata);
+            return NHF_NEUTRAL;
+        }
     }
 
     if ((status = enter_grouping(loc_id, name, H5O_TYPE_DATASET, "dataset_%04u", 12,
