@@ -1,5 +1,5 @@
 /*
- *  $Id: nsidfile.c 26873 2024-11-19 12:52:14Z yeti-dn $
+ *  $Id: nsidfile.c 28533 2025-09-10 14:07:50Z yeti-dn $
  *  Copyright (C) 2024 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -103,15 +103,13 @@ look_for_canary_attributes(hid_t loc_id, guint *flags,
         { "modality",       FOUND_MODALITY,       },
         { "data_type",      FOUND_DATA_TYPE,      },
     };
-    hid_t attr;
     guint i, bit;
 
     for (i = 0; i < G_N_ELEMENTS(attributes); i++) {
         bit = attributes[i].value;
-        if (!(*flags & bit) && (attr = H5Aopen(loc_id, attributes[i].name, H5P_DEFAULT)) >= 0) {
+        if (!(*flags & bit) && H5Aexists(loc_id, attributes[i].name) >= 0) {
             gwy_debug("found %s (at depth %d)", attributes[i].name, depth);
             *flags |= bit;
-            H5Aclose(attr);
         }
     }
 }
