@@ -1,5 +1,5 @@
 /*
- *  $Id: shilpsfile.c 26873 2024-11-19 12:52:14Z yeti-dn $
+ *  $Id: shilpsfile.c 28533 2025-09-10 14:07:50Z yeti-dn $
  *  Copyright (C) 2020-2024 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -147,7 +147,8 @@ shilps_detect(const GwyFileDetectInfo *fileinfo,
     if ((file_id = gwyhdf5_quick_check(fileinfo, only_name)) < 0)
         return 0;
 
-    if (gwyhdf5_get_str_attr(file_id, ".", "Company", &company, NULL)) {
+    if (H5Aexists(file_id, "Company")
+        && gwyhdf5_get_str_attr(file_id, ".", "Company", &company, NULL)) {
         if (gwy_strequal(company, "Shilps Sciences"))
             score = 100;
         H5free_memory(company);
