@@ -1,5 +1,5 @@
 /*
- *  $Id: jpkscan.c 28204 2025-07-07 16:29:29Z yeti-dn $
+ *  $Id: jpkscan.c 28413 2025-08-14 18:17:44Z yeti-dn $
  *  Loader for JPK Image Scans.
  *  Copyright (C) 2005  JPK Instruments AG.
  *  Written by Sven Neumann <neumann@jpk.com>.
@@ -87,6 +87,12 @@ typedef enum {
     JPK_FORCE_QI,
 } JPKForceFileType;
 
+typedef enum {
+    JPK_PAUSE_UNKNOWN = 0,
+    JPK_PAUSE_IGNORED = -1,
+    JPK_PAUSE_INCLUDED = 1
+} JPKPauseHandling;
+
 typedef struct {
     /* Segment header properties. */
     /* Concatenated data of all channels. */
@@ -113,6 +119,7 @@ typedef struct {
     GHashTable *header_properties;
     GHashTable *shared_header_properties;
     JPKForceFileType type;
+    JPKPauseHandling pause_handling;
     guint nids;
     guint *ids;
     guint nsegs;
@@ -277,7 +284,7 @@ static GwyModuleInfo module_info = {
     module_register,
     N_("Imports JPK image scans."),
     "Sven Neumann <neumann@jpk.com>, Yeti <yeti@gwyddion.net>",
-    "0.17",
+    "0.18",
     "JPK Instruments AG, David Nečas (Yeti)",
     "2005-2007",
 };
@@ -657,6 +664,7 @@ jpkforce_load(const gchar *filename,
     jpkfile.index_regex = g_regex_new("^index/([0-9]+)/(.*)$", G_REGEX_OPTIMIZE, 0, NULL);
     jpkfile.index_segment_regex = g_regex_new("^index/([0-9]+)/segments/([0-9]+)/(.*)$", G_REGEX_OPTIMIZE, 0, NULL);
     jpkfile.filename = filename;
+    jpkfile.pause_handling = JPK_PAUSE_UNKNOWN;
 
     gwy_debug("starting scanning");
     if (mode == GWY_RUN_INTERACTIVE) {
@@ -1113,8 +1121,7 @@ read_curve_data(GwyZipFile zipfile, JPKForceFile *jpkfile, GError **error)
             }
             g_free(filename);
 
-            /* Read the data, unless it is a pause segment, then do not
-             * bother. */
+            /* Read the data, unless it is a pause segment, then do not bother. */
             if (!is_pause) {
                 if (!read_raw_data(zipfile, jpkfile, data, hash, datatype, 0, cid, ndata, error))
                     return FALSE;
@@ -1231,23 +1238,35 @@ create_curve_map(GwyContainer *container, JPKForceFile *jpkfile,
                  GError **error)
 {
     guint sid, cid, height_cid = jpkfile->height_cid;
-    guint kpt, ij, nchannels = jpkfile->nchannels, nsegs = jpkfile->nsegs;
+    guint kpt, i, ij, nchannels = jpkfile->nchannels, nsegs = jpkfile->nsegs, real_nsegs;
     guint xres = jpkfile->xres, yres = jpkfile->yres;
     guint blocklen;
     GwyLawn *lawn;
     const gdouble *segdata;
     GArray *curvesdata;
     gint *segments;
+    gboolean *included_segments;
 
     g_return_val_if_fail(height_cid < jpkfile->nchannels, 0);
 
-    lawn = gwy_lawn_new(xres, yres, jpkfile->xystep.x*xres, jpkfile->xystep.y*yres, nchannels, nsegs);
+    included_segments = g_new(gboolean, nsegs);
+    real_nsegs = 0;
+    for (sid = 0; sid < nsegs; sid++) {
+        included_segments[sid] = TRUE;
+        if (jpkfile->pause_handling == JPK_PAUSE_IGNORED) {
+            JPKForceData *data = jpkfile->data + sid;
+            included_segments[sid] = !(data->segment_style && gwy_strequal(data->segment_style, "pause"));
+        }
+        real_nsegs += !!included_segments[sid];
+    }
+
+    lawn = gwy_lawn_new(xres, yres, jpkfile->xystep.x*xres, jpkfile->xystep.y*yres, nchannels, real_nsegs);
     gwy_si_unit_set_from_string(gwy_lawn_get_si_unit_xy(lawn), "m");
     gwy_lawn_set_xoffset(lawn, jpkfile->xyorigin.x);
     gwy_lawn_set_yoffset(lawn, jpkfile->xyorigin.y);
 
     curvesdata = g_array_new(FALSE, FALSE, sizeof(gdouble));
-    segments = g_new(gint, 2*nsegs);
+    segments = g_new(gint, 2*real_nsegs);
 
     /* Use the first force data set for units.
      * FIXME: Do we ensure the segments have uniform channel sets somewhere? We have just one set of channel names,
@@ -1256,8 +1275,12 @@ create_curve_map(GwyContainer *container, JPKForceFile *jpkfile,
         gwy_si_unit_set_from_string(gwy_lawn_get_si_unit_curve(lawn, cid), jpkfile->data->units[cid]);
         gwy_lawn_set_curve_label(lawn, cid, jpkfile->channel_names[cid]);
     }
-    for (sid = 0; sid < nsegs; sid++)
-        gwy_lawn_set_segment_label(lawn, sid, jpkfile->data[sid].segment_name);
+
+    i = 0;
+    for (sid = 0; sid < nsegs; sid++) {
+        if (included_segments[sid])
+            gwy_lawn_set_segment_label(lawn, i++, jpkfile->data[sid].segment_name);
+    }
 
     for (ij = 0; ij < xres*yres; ij++) {
         g_array_set_size(curvesdata, 0);
@@ -1266,14 +1289,18 @@ create_curve_map(GwyContainer *container, JPKForceFile *jpkfile,
         /* Gather data of all curves for the pixel. */
         for (cid = 0; cid < nchannels; cid++) {
             /* Gather data of all segments. */
+            i = 0;
             for (sid = 0; sid < nsegs; sid++) {
                 JPKForceData *data = jpkfile->data + sid;
 
+                if (!included_segments[sid])
+                    continue;
                 segdata = data->data + data->ndata*(cid + kpt*nchannels);
                 blocklen = data->measured_ndata[kpt];
                 if (!cid) {
-                    segments[2*sid] = curvesdata->len/2;
-                    segments[2*sid + 1] = (curvesdata->len + blocklen)/2;
+                    segments[2*i] = curvesdata->len;
+                    segments[2*i + 1] = (curvesdata->len + blocklen);
+                    i++;
                 }
                 g_array_append_vals(curvesdata, segdata, blocklen);
             }
@@ -1294,6 +1321,7 @@ create_curve_map(GwyContainer *container, JPKForceFile *jpkfile,
     gwy_container_pass_object(container, gwy_app_get_lawn_key_for_id(0), lawn);
 
 fail:
+    g_free(included_segments);
     g_free(segments);
     g_array_free(curvesdata, TRUE);
 
@@ -1436,7 +1464,6 @@ static gboolean
 read_forcemap_data(GwyZipFile zipfile, JPKForceFile *jpkfile,
                    GwySetFractionFunc set_fraction, GError **error)
 {
-    enum { PAUSE_UNKNOWN = 0, PAUSE_IGNORED = -1, PAUSE_INCLUDED = 1 } pause_handling = PAUSE_UNKNOWN;
     GHashTable *hash;
     JPKForceData *data;
     GString *str = jpkfile->str;
@@ -1553,7 +1580,7 @@ read_forcemap_data(GwyZipFile zipfile, JPKForceFile *jpkfile,
         /* XXX: Pause segments can have different channels. Frequently they have no channels and no data files. In
          * such case we ignore them and must not try to read any data. Initially, we do not know, so decide when we
          * first encounter a pause segment. */
-        if (is_pause && pause_handling == PAUSE_IGNORED)
+        if (is_pause && jpkfile->pause_handling == JPK_PAUSE_IGNORED)
             goto finish_map_point;
 
         if (!enumerate_channels(jpkfile, hash, TRUE, error) || (is_pause && !jpkfile->nchannels)) {
@@ -1563,11 +1590,11 @@ read_forcemap_data(GwyZipFile zipfile, JPKForceFile *jpkfile,
              *
              * We need to handle the odd case when the very first segment we encounter is a bogus pause segment which
              * should not be used as the template for others. i.e. make sure jpkfile->channel_names remains unset. */
-            if (!is_pause || pause_handling == PAUSE_INCLUDED)
+            if (!is_pause || jpkfile->pause_handling == JPK_PAUSE_INCLUDED)
                 return FALSE;
             g_clear_error(error);
-            if (pause_handling == PAUSE_UNKNOWN) {
-                pause_handling = PAUSE_IGNORED;
+            if (jpkfile->pause_handling == JPK_PAUSE_UNKNOWN) {
+                jpkfile->pause_handling = JPK_PAUSE_IGNORED;
                 gwy_debug("pause segments are IGNORED now");
                 if (!jpkfile->nchannels) {
                     GWY_FREE(jpkfile->channel_names);
@@ -1578,9 +1605,9 @@ read_forcemap_data(GwyZipFile zipfile, JPKForceFile *jpkfile,
         }
 
         /* A pause segment has valid channels. So from now on assume pause segments contain real data. */
-        if (is_pause && pause_handling == PAUSE_UNKNOWN) {
+        if (is_pause && jpkfile->pause_handling == JPK_PAUSE_UNKNOWN) {
             gwy_debug("pause segments are INCLUDED now");
-            pause_handling = PAUSE_INCLUDED;
+            jpkfile->pause_handling = JPK_PAUSE_INCLUDED;
         }
 
         if (!(ndata = find_segment_npoints(jpkfile, hash, error)))
