@@ -1,5 +1,5 @@
 /*
- *  $Id: lextfile.c 26182 2024-02-16 16:06:42Z yeti-dn $
+ *  $Id: lextfile.c 28937 2025-12-02 15:09:01Z yeti-dn $
  *  Copyright (C) 2010-2019 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -83,7 +83,7 @@ static GwyModuleInfo module_info = {
     module_register,
     N_("Imports LEXT data files."),
     "Yeti <yeti@gwyddion.net>",
-    "0.6",
+    "0.6.1",
     "David Nečas (Yeti) & Petr Klapetek",
     "2010",
 };
@@ -234,7 +234,7 @@ lext_load_tiff(const GwyTIFF *tiff, const gchar *filename, GError **error)
     gchar *comment = NULL, *title = NULL;
     const gchar *value, *image0title, *keytitle, *channeltitle, *unitstr;
     GError *err = NULL;
-    guint dir_num = 0, ch, spp;
+    guint dir_num2 = 0, ndirs, ch, spp;
     gint id = 0;
     GString *key;
 
@@ -263,7 +263,11 @@ lext_load_tiff(const GwyTIFF *tiff, const gchar *filename, GError **error)
     image0title = guess_image0_title(tiff);
     gwy_debug("guessed image0 type: %s", image0title ? image0title : "UNKNOWN");
 
-    for (dir_num = 0; dir_num < gwy_tiff_get_n_dirs(tiff); dir_num++) {
+    /* Go through the directories twice. Extract "Height" during the first pass and everything else during the second
+     * one. */
+    ndirs = gwy_tiff_get_n_dirs(tiff);
+    for (dir_num2 = 0; dir_num2 < 2*ndirs; dir_num2++) {
+        gint dir_num = dir_num2 % ndirs;
         double xscale, yscale, zfactor;
         GQuark quark;
         gdouble *data;
@@ -292,6 +296,14 @@ lext_load_tiff(const GwyTIFF *tiff, const gchar *filename, GError **error)
         if (gwy_stramong(title, "Thumbnail", "Invalid", NULL))
             continue;
 
+        /* Only "Height" in the first pass. */
+        if (gwy_strequal(title, "Height")) {
+            if (dir_num2 >= ndirs)
+                continue;
+        }
+        else if (dir_num2 < ndirs)
+            continue;
+
         if (gwy_strequal(title, "Color"))
             keytitle = "Intensity";
         else
