@@ -1,5 +1,5 @@
 /*
- *  $Id: rawfile.c 26744 2024-10-18 15:03:29Z yeti-dn $
+ *  $Id: rawfile.c 28628 2025-09-30 12:17:05Z yeti-dn $
  *  Copyright (C) 2003-2022 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -100,9 +100,11 @@ enum {
     PARAM_XYEXPONENT,
     PARAM_ZEXPONENT,
     PARAM_FORMAT,
-    PARAM_LINEOFFSET,
+    PARAM_LINE_OFFSET,
     PARAM_BYTESWAP,
-    PARAM_SKIPFIELDS,
+    PARAM_SKIP_FIELDS,
+    PARAM_ONE_PER_LINE,
+    PARAM_COLUMN,
     PARAM_DELIMITER,
     PARAM_DECOMMA,
     PARAM_BUILTIN,
@@ -163,6 +165,7 @@ typedef struct {
     GwyParams *params;
     RawFileFile file;
     GwyDataField *field;
+    GwyDataField *mask;
 } ModuleArgs;
 
 typedef struct {
@@ -254,7 +257,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Imports raw data files, both ASCII and binary, according to user-specified format."),
     "Yeti <yeti@gwyddion.net>",
-    "3.1",
+    "3.3",
     "David Nečas (Yeti) & Petr Klapetek",
     "2003",
 };
@@ -355,8 +358,10 @@ add_preset_params(GwyParamDef *paramdef)
     gwy_param_def_add_gwyenum(paramdef, PARAM_FORMAT, "format", NULL, formats, G_N_ELEMENTS(formats), RAW_BINARY);
 
     /* Text. */
-    gwy_param_def_add_int(paramdef, PARAM_LINEOFFSET, "lineoffset", _("Start from _line"), 0, 1 << 28, 0);
-    gwy_param_def_add_int(paramdef, PARAM_SKIPFIELDS, "skipfields", _("_Each row skip"), 0, 1 << 28, 0);
+    gwy_param_def_add_int(paramdef, PARAM_LINE_OFFSET, "lineoffset", _("Start from file _line"), 0, 1 << 28, 0);
+    gwy_param_def_add_int(paramdef, PARAM_SKIP_FIELDS, "skipfields", _("_Each image row skip"), 0, 1 << 28, 0);
+    gwy_param_def_add_boolean(paramdef, PARAM_ONE_PER_LINE, "oneperline", _("One _value per line"), FALSE);
+    gwy_param_def_add_int(paramdef, PARAM_COLUMN, "column", _("E_xtract column"), 1, 1 << 10, 1);
     gwy_param_def_add_string(paramdef, PARAM_DELIMITER, "delimiter", _("_Other delimiter"),
                              GWY_PARAM_STRING_NULL_IS_EMPTY | GWY_PARAM_STRING_DO_NOT_STRIP, NULL, "");
     gwy_param_def_add_boolean(paramdef, PARAM_DECOMMA, "decomma", _("_Decimal separator is comma"), FALSE);
@@ -393,7 +398,6 @@ rawfile_load(const gchar *filename,
     ModuleArgs args;
     GwyContainer *data = NULL;
     GwyDialogOutcome outcome;
-    GwyDataField *mask;
     GError *err = NULL;
     gsize size = 0;
 
@@ -425,9 +429,8 @@ rawfile_load(const gchar *filename,
 
     data = gwy_container_new();
     gwy_container_set_object(data, gwy_app_get_data_key_for_id(0), args.field);
-    if ((mask = gwy_app_channel_mask_of_nans(args.field, TRUE))) {
-        gwy_container_pass_object(data, gwy_app_get_mask_key_for_id(0), mask);
-    }
+    if (args.mask)
+        gwy_container_pass_object(data, gwy_app_get_mask_key_for_id(0), args.mask);
     gwy_file_channel_import_log_add(data, 0, NULL, filename);
 
 end:
@@ -575,11 +578,14 @@ format_tab_new(ModuleGUI *gui)
 
     /* Text */
     gwy_param_table_append_radio_item(table, PARAM_FORMAT, RAW_TEXT);
-    gwy_param_table_append_slider(table, PARAM_LINEOFFSET);
-    gwy_param_table_slider_set_mapping(table, PARAM_LINEOFFSET, GWY_SCALE_MAPPING_LOG1P);
-    gwy_param_table_append_slider(table, PARAM_SKIPFIELDS);
-    gwy_param_table_slider_set_mapping(table, PARAM_SKIPFIELDS, GWY_SCALE_MAPPING_LOG1P);
-    gwy_param_table_set_unitstr(table, PARAM_SKIPFIELDS, _("fields"));
+    gwy_param_table_append_slider(table, PARAM_LINE_OFFSET);
+    gwy_param_table_slider_set_mapping(table, PARAM_LINE_OFFSET, GWY_SCALE_MAPPING_LOG1P);
+    gwy_param_table_append_slider(table, PARAM_SKIP_FIELDS);
+    gwy_param_table_slider_set_mapping(table, PARAM_SKIP_FIELDS, GWY_SCALE_MAPPING_LOG1P);
+    gwy_param_table_set_unitstr(table, PARAM_SKIP_FIELDS, _("fields"));
+    gwy_param_table_append_checkbox(table, PARAM_ONE_PER_LINE);
+    gwy_param_table_append_slider(table, PARAM_COLUMN);
+    gwy_param_table_slider_set_mapping(table, PARAM_COLUMN, GWY_SCALE_MAPPING_LOG1P);
     gwy_param_table_append_combo(table, PARAM_DELIMITER_TYPE);
     gwy_param_table_append_entry(table, PARAM_DELIMITER);
     gwy_param_table_append_checkbox(table, PARAM_DECOMMA);
@@ -753,7 +759,8 @@ static void
 param_changed(ModuleGUI *gui, gint id)
 {
     static const guint text_params[] = {
-        PARAM_LINEOFFSET, PARAM_SKIPFIELDS, PARAM_DELIMITER_TYPE, PARAM_DELIMITER, PARAM_DECOMMA,
+        PARAM_LINE_OFFSET, PARAM_SKIP_FIELDS, PARAM_ONE_PER_LINE, PARAM_COLUMN,
+        PARAM_DELIMITER_TYPE, PARAM_DELIMITER, PARAM_DECOMMA,
     };
     static const guint binary_params[] = {
         PARAM_BUILTIN, PARAM_BYTESWAP, PARAM_OFFSET, PARAM_SIZE, PARAM_SKIP, PARAM_ROWSKIP, PARAM_REVBYTE,
@@ -789,6 +796,10 @@ param_changed(ModuleGUI *gui, gint id)
     if (format == RAW_TEXT) {
         gwy_param_table_set_sensitive(gui->table_format, PARAM_DELIMITER,
                                       gwy_params_get_enum(params, PARAM_DELIMITER_TYPE) == RAW_DELIM_OTHER);
+        gwy_param_table_set_sensitive(gui->table_format, PARAM_SKIP_FIELDS,
+                                      !gwy_params_get_boolean(params, PARAM_ONE_PER_LINE));
+        gwy_param_table_set_sensitive(gui->table_format, PARAM_COLUMN,
+                                      gwy_params_get_boolean(params, PARAM_ONE_PER_LINE));
     }
 
     /* FIXME: Previously, switching from user to builtin and back restored the user format size and sign. */
@@ -857,7 +868,7 @@ param_changed(ModuleGUI *gui, gint id)
     }
 
     if (id < 0 || id == PARAM_FORMAT || id == PARAM_XRES || id == PARAM_YRES
-        || id == PARAM_SKIPFIELDS || id == PARAM_LINEOFFSET || id == PARAM_DELIMITER || id == PARAM_DELIMITER_TYPE
+        || id == PARAM_SKIP_FIELDS || id == PARAM_LINE_OFFSET || id == PARAM_DELIMITER || id == PARAM_DELIMITER_TYPE
         || id == PARAM_BUILTIN || id == PARAM_SIZE || id == PARAM_SKIP || id == PARAM_ROWSKIP || id == PARAM_OFFSET)
         gtk_label_set_text(GTK_LABEL(gui->error), "");
 }
@@ -867,13 +878,13 @@ preview(gpointer user_data)
 {
     ModuleGUI *gui = (ModuleGUI*)user_data;
     ModuleArgs *args = gui->args;
-    GwyDataField *field, *mask;
+    GwyDataField *field;
     GError *error = NULL;
 
     if ((field = read_data_field(args, &error))) {
-        /* Fix any NaNs in the data for preview. */
-        if ((mask = gwy_app_channel_mask_of_nans(field, TRUE)))
-            g_object_unref(mask);
+        /* Fix any NaNs in the data for preview and remember the mask for later. */
+        g_clear_object(&args->mask);
+        args->mask = gwy_app_channel_mask_of_nans(field, TRUE);
         gwy_data_field_assign(args->field, field);
         g_object_unref(field);
         gtk_label_set_text(GTK_LABEL(gui->error), "");
@@ -1058,10 +1069,16 @@ static gboolean
 missingval_is_number(const gchar *missingval)
 {
     gchar *end;
+    gdouble v;
 
-    g_strtod(missingval, &end);
+    v = g_strtod(missingval, &end);
     if (end == missingval)
         return FALSE;
+    /* If people do not check the missing value checkbox at all, NaN literals generally work fine. If they do, only
+     * consider actual numbers to be numbers. If they say ‘NAN’, expect literal strings ‘NAN’. It behaves weird when
+     * we return TRUE for NaNs here. */
+    if (gwy_isinf(v) || gwy_isnan(v))
+        return FALSE;
 
     return TRUE;
 }
@@ -1081,9 +1098,9 @@ read_data_field(ModuleArgs *args, GError **error)
     guint yres = gwy_params_get_int(params, PARAM_YRES);
     gulong skip = gwy_params_get_int(params, PARAM_SKIP);
     gulong rowskip = gwy_params_get_int(params, PARAM_ROWSKIP);
-    gulong skipfields = gwy_params_get_int(params, PARAM_SKIPFIELDS);
+    gulong skipfields = gwy_params_get_int(params, PARAM_SKIP_FIELDS);
     gulong offset = gwy_params_get_int(params, PARAM_OFFSET);
-    gulong lineoffset = gwy_params_get_int(params, PARAM_LINEOFFSET);
+    gulong lineoffset = gwy_params_get_int(params, PARAM_LINE_OFFSET);
     gdouble xreal = gwy_params_get_double(params, PARAM_XREAL);
     gdouble yreal = gwy_params_get_double(params, PARAM_YREAL);
     GwyDataField *field;
@@ -1475,25 +1492,160 @@ rawfile_read_builtin(ModuleArgs *args, gdouble *data)
     g_free(rtable8);
 }
 
+static const guchar*
+skip_fields(const guchar *buffer, gint nfields,
+            gint delimlen, gint cdelim, const gchar *delim,
+            gint rowno, GError **error)
+{
+    const gchar *end;
+    gint i, j;
+
+    if (!nfields)
+        return buffer;
+
+    if (!delimlen) {
+        buffer += strspn(buffer, " \t\n\r");
+        for (i = 0; i < nfields; i++) {
+            j = strcspn(buffer, " \t\n\r");
+            buffer += j;
+            j = strspn(buffer, " \t\n\r");
+            if (!j) {
+                g_set_error(error, error_domain, RAWFILE_ERROR_TEXT_PARSING,
+                            _("Expected whitespace to skip more fields in row %u, got `%.16s'"),
+                            rowno, buffer);
+                return NULL;
+            }
+            buffer += j;
+        }
+    }
+    else if (delimlen == 1) {
+        for (i = 0; i < nfields; i++) {
+            end = strchr(buffer, cdelim);
+            if (!end) {
+                g_set_error(error, error_domain, RAWFILE_ERROR_TEXT_PARSING,
+                            _("Expected `%c' to skip more fields in row %u, got `%.16s'"),
+                            cdelim, rowno, buffer);
+                return NULL;
+            }
+            buffer = end + 1;
+        }
+    }
+    else {
+        for (i = 0; i < nfields; i++) {
+            end = strstr(buffer, delim);
+            if (!end) {
+                g_set_error(error, error_domain, RAWFILE_ERROR_TEXT_PARSING,
+                            _("Expected `%s' to skip more fields in row %u, got `%.16s'"),
+                            delim, rowno, buffer);
+                return NULL;
+            }
+            buffer = end + delimlen;
+        }
+    }
+    return buffer;
+}
+
+static const guchar*
+read_fields(const guchar *buffer, gint nfields,
+            gint delimlen, gint cdelim, const gchar *delim,
+            RawStrtodFunc strtod_func, const gchar *missingval,
+            gdouble *data,
+            gint rowno, GError **error)
+{
+    gboolean single_value = !nfields;
+    const guchar *end;
+    gdouble x;
+    gint i, j;
+
+    nfields = MAX(nfields, 1);
+    if (!delimlen) {
+        for (i = 0; i < nfields; i++) {
+            /* FIXME: This is not good enough when extracting a single value. The strtod function can also accept
+             * \n and \r as whitespace, messing up the ‘one per line’ paradigm. If the data sort of match what the
+             * user says, it works fine. If they do not the results are mysterious. */
+            x = strtod_func(buffer, missingval, (char**)&end);
+            if (end == buffer) {
+                g_set_error(error, error_domain, RAWFILE_ERROR_TEXT_PARSING,
+                            _("Garbage `%.16s' in row %u, column %u"),
+                            buffer, rowno, i);
+                return NULL;
+            }
+            buffer = end;
+            *(data++) = x;
+        }
+    }
+    else if (delimlen == 1) {
+        for (i = 0; i < nfields; i++) {
+            x = strtod_func(buffer, missingval, (char**)&end);
+            if (end == buffer) {
+                g_set_error(error, error_domain, RAWFILE_ERROR_TEXT_PARSING,
+                            _("Garbage `%.16s' in row %u, column %u"),
+                            buffer, rowno, i);
+                return NULL;
+            }
+            buffer = end + strspn(end, " \t");
+            if (*buffer == cdelim)
+                buffer++;
+            else if (i + 1 == nfields && (j = strspn(buffer, "\n\r"))) {
+                if (!single_value)
+                    buffer += j;
+            }
+            else {
+                g_set_error(error, error_domain, RAWFILE_ERROR_TEXT_PARSING,
+                            _("Expected delimiter `%c' after data in row %u, column %u, got `%c'"),
+                            cdelim, rowno, i, *buffer);
+                return NULL;
+            }
+            *(data++) = x;
+        }
+    }
+    else {
+        for (i = 0; i < nfields; i++) {
+            x = strtod_func(buffer, missingval, (char**)&end);
+            if (end == buffer) {
+                g_set_error(error, error_domain, RAWFILE_ERROR_TEXT_PARSING,
+                            _("Garbage `%.16s' in row %u, column %u"),
+                            buffer, rowno, i);
+                return NULL;
+            }
+            buffer = end + strspn(end, " \t");
+            if (strncmp(buffer, delim, delimlen) == 0)
+                buffer += delimlen;
+            else if (i + 1 == nfields && (j = strspn(buffer, "\n\r"))) {
+                if (!single_value)
+                    buffer += j;
+            }
+            else {
+                g_set_error(error, error_domain, RAWFILE_ERROR_TEXT_PARSING,
+                            _("Expected delimiter `%s' after data in row %u, column %u, got `%.16s'"),
+                            delim, rowno, i, buffer);
+                return NULL;
+            }
+            *(data++) = x;
+        }
+    }
+    return buffer;
+}
+
 static gboolean
 rawfile_read_ascii(ModuleArgs *args, gdouble *data, GError **error)
 {
     GwyParams *params = args->params;
     guint xres = gwy_params_get_int(params, PARAM_XRES);
     guint yres = gwy_params_get_int(params, PARAM_YRES);
-    guint lineoffset = gwy_params_get_int(params, PARAM_LINEOFFSET);
-    guint skipfields = gwy_params_get_int(params, PARAM_SKIPFIELDS);
+    guint lineoffset = gwy_params_get_int(params, PARAM_LINE_OFFSET);
+    guint skipfields = gwy_params_get_int(params, PARAM_SKIP_FIELDS);
+    guint column = gwy_params_get_int(params, PARAM_COLUMN);
     const gchar *delim = gwy_params_get_string(params, PARAM_DELIMITER);
     const gchar *missingval = gwy_params_get_string(params, PARAM_MISSING_VALUE);
     gboolean havemissing = gwy_params_get_boolean(params, PARAM_HAVE_MISSING);
     gboolean decomma = gwy_params_get_boolean(params, PARAM_DECOMMA);
+    gboolean oneperline = gwy_params_get_boolean(params, PARAM_ONE_PER_LINE);
     RawStrtodFunc strtod_func = (decomma ? gwy_comma_strtod : gwy_ascii_strtod);
     const guchar *buffer = args->file.buffer;
-    guint i, j, n;
+    guint i, n;
     gint cdelim = '\0';
     gint delimlen;
-    gdouble x;
-    guchar *end;
 
     if (!havemissing || missingval_is_number(missingval))
         missingval = NULL;
@@ -1524,109 +1676,33 @@ rawfile_read_ascii(ModuleArgs *args, gdouble *data, GError **error)
         cdelim = delim[0];
     }
 
-    for (n = 0; n < yres; n++) {
-        /* Skip fields. */
-        if (!delimlen) {
-            buffer += strspn(buffer, " \t\n\r");
-            for (i = 0; i < skipfields; i++) {
-                j = strcspn(buffer, " \t\n\r");
-                buffer += j;
-                j = strspn(buffer, " \t\n\r");
-                if (!j) {
-                    g_set_error(error, error_domain, RAWFILE_ERROR_TEXT_PARSING,
-                                _("Expected whitespace to skip more fields in row %u, got `%.16s'"),
-                                n, buffer);
-                    return FALSE;
-                }
-                buffer += j;
-            }
-        }
-        else if (delimlen == 1) {
-            for (i = 0; i < skipfields; i++) {
-                end = strchr(buffer, cdelim);
-                if (!end) {
-                    g_set_error(error, error_domain, RAWFILE_ERROR_TEXT_PARSING,
-                                _("Expected `%c' to skip more fields in row %u, got `%.16s'"),
-                                cdelim, n, buffer);
-                    return FALSE;
-                }
-                buffer = end + 1;
-            }
-        }
-        else {
-            for (i = 0; i < skipfields; i++) {
-                end = strstr(buffer, delim);
-                if (!end) {
-                    g_set_error(error, error_domain, RAWFILE_ERROR_TEXT_PARSING,
-                                _("Expected `%s' to skip more fields in row %u, got `%.16s'"),
-                                delim, n, buffer);
-                    return FALSE;
-                }
-                buffer = end + delimlen;
-            }
-        }
+    if (oneperline) {
+        /* Extract one value from each file line. */
+        for (n = 0; n < xres*yres; n++) {
+            /* Skip fields. */
+            if (!(buffer = skip_fields(buffer, column-1, delimlen, cdelim, delim, n, error)))
+                return FALSE;
 
-        /* Read data. */
-        if (!delimlen) {
-            for (i = 0; i < xres; i++) {
-                x = strtod_func(buffer, missingval, (char**)&end);
-                if (end == buffer) {
-                    g_set_error(error, error_domain, RAWFILE_ERROR_TEXT_PARSING,
-                                _("Garbage `%.16s' in row %u, column %u"),
-                                buffer, n, i);
-                    return FALSE;
-                }
-                buffer = end;
-                *(data++) = x;
-            }
+            /* Read a value. */
+            if (!(buffer = read_fields(buffer, 0, delimlen, cdelim, delim, strtod_func, missingval, data, n, error)))
+                return FALSE;
+            data++;
+
+            /* Skip to the end of line (and skip any empty lines). */
+            buffer += strcspn(buffer, "\r\n");
         }
-        else if (delimlen == 1) {
-            for (i = 0; i < xres; i++) {
-                x = strtod_func(buffer, missingval, (char**)&end);
-                if (end == buffer) {
-                    g_set_error(error, error_domain, RAWFILE_ERROR_TEXT_PARSING,
-                                _("Garbage `%.16s' in row %u, column %u"),
-                                buffer, n, i);
-                    return FALSE;
-                }
-                buffer = end + strspn(end, " \t");
-                if (*buffer == cdelim)
-                    buffer++;
-                else if (i + 1 == xres
-                         && (j = strspn(buffer, "\n\r")))
-                    buffer += j;
-                else {
-                    g_set_error(error, error_domain, RAWFILE_ERROR_TEXT_PARSING,
-                                _("Expected delimiter `%c' after data in row %u, column %u, got `%c'"),
-                                cdelim, n, i, *buffer);
-                    return FALSE;
-                }
-                *(data++) = x;
-            }
-        }
-        else {
-            for (i = 0; i < xres; i++) {
-                x = strtod_func(buffer, missingval, (char**)&end);
-                if (end == buffer) {
-                    g_set_error(error, error_domain, RAWFILE_ERROR_TEXT_PARSING,
-                                _("Garbage `%.16s' in row %u, column %u"),
-                                buffer, n, i);
-                    return FALSE;
-                }
-                buffer = end + strspn(end, " \t");
-                if (strncmp(buffer, delim, delimlen) == 0)
-                    buffer += delimlen;
-                else if (i + 1 == xres
-                         && (j = strspn(buffer, "\n\r")))
-                    buffer += j;
-                else {
-                    g_set_error(error, error_domain, RAWFILE_ERROR_TEXT_PARSING,
-                                _("Expected delimiter `%s' after data in row %u, column %u, got `%.16s'"),
-                                delim, n, i, buffer);
-                    return FALSE;
-                }
-                *(data++) = x;
-            }
+    }
+    else {
+        /* Free-form reading using the given delimiter. */
+        for (n = 0; n < yres; n++) {
+            /* Skip fields. */
+            if (!(buffer = skip_fields(buffer, skipfields, delimlen, cdelim, delim, n, error)))
+                return FALSE;
+
+            /* Read data. */
+            if (!(buffer = read_fields(buffer, xres, delimlen, cdelim, delim, strtod_func, missingval, data, n, error)))
+                return FALSE;
+            data += xres;
         }
     }
 
