@@ -1,5 +1,5 @@
 /*
- *  $Id: tiaser.c 25795 2023-10-06 15:20:36Z yeti-dn $
+ *  $Id: tiaser.c 28573 2025-09-17 13:55:53Z yeti-dn $
  *  Copyright (C) 2012 David Necas (Yeti), Daniil Bratashov (dn2010).
  *  E-mail: yeti@gwyddion.net, dn2010@gmail.com.
  *
@@ -304,13 +304,7 @@ tia_load(const gchar *filename,
     TIADimension *dimension, dim;
     GArray *dimarray, *dataoffsets, *tagoffsets;
     GwyDataField *dfield;
-    GwyBrick *brick;
-    GwyDataLine *dline;
-    GwySIUnit *siunit;
-    gint i, k, offset, dimarraylength, dimarraysize = 0;
-    gint xres, yres, zres, xpos, ypos;
-    gdouble xreal, xoffset, yreal, yoffset, zreal, zoffset;
-    gdouble *value, *data;
+    gint i, offset, dimarraylength, dimarraysize = 0;
     gchar *strkey;
 
     if (!gwy_file_get_contents(filename, &buffer, &size, &err)) {
@@ -385,6 +379,19 @@ tia_load(const gchar *filename,
             }
         }
     else if (header->datatypeid == TIA_1D_DATA) {
+        /* XXX XXX XXX: What if the sizes of individual lines differ from the brick zres? This is just one big fucking
+         * buffer overflow waiting to happen! If each has an independent length we should be probably creating a Lawn
+         * anyway.
+         *
+         * Disabled for now. */
+#if 0
+        GwyBrick *brick;
+        GwyDataLine *dline;
+        GwySIUnit *siunit;
+        gint xres, yres, zres, xpos, ypos;
+        gdouble xreal, xoffset, yreal, yoffset, zreal, zoffset;
+        gdouble *value, *data;
+
         if (dimarray->len != 2) {
             gwy_debug("Wrong dimensions number");
             goto tagoffsets_fail;
@@ -457,6 +464,7 @@ tia_load(const gchar *filename,
             gwy_container_set_const_string_by_name(container, "/brick/0/title", "TEM Spectroscopy");
             gwy_file_volume_import_log_add(container, 0, NULL, filename);
         }
+#endif
     }
 
 tagoffsets_fail:
