@@ -1,5 +1,5 @@
 /*
- *  $Id: neural.c 26747 2024-10-18 15:23:06Z yeti-dn $
+ *  $Id: neural.c 28641 2025-10-12 07:13:26Z klapetek $
  *  Copyright (C) 2012-2024 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -525,7 +525,7 @@ training_tab_new(NeuralTrainGUI *gui)
     gwy_param_table_append_slider(table, PARAM_TRAINSTEPS);
     gwy_param_table_append_separator(table);
     gwy_param_table_append_radio(table, PARAM_PREVIEW);
-    gwy_param_table_append_button(table, BUTTON_TRAIN, -1, RESPONSE_TRAIN, _("verb|_Train"));
+    gwy_param_table_append_button(table, BUTTON_TRAIN, -1, RESPONSE_TRAIN, gwy_sgettext("verb|_Train"));
     gwy_param_table_append_button(table, BUTTON_REINITIALIZE, BUTTON_TRAIN, RESPONSE_REINITIALIZE, _("Re_initialize"));
     gwy_param_table_append_message(table, MESSAGE_TRAINING, NULL);
     gwy_param_table_append_separator(table);
@@ -1224,6 +1224,8 @@ ensure_network(GwyParams *params)
 {
     GwyNeuralNetwork *nn;
 
+    printf("ensstring %s\n", gwy_params_get_string(params, PARAM_NAME));
+
     if ((nn = gwy_inventory_get_item(gwy_neural_networks(), gwy_params_get_string(params, PARAM_NAME))))
         return nn;
 
