@@ -137,6 +137,7 @@ GwyModuleInfo* _gwy_module_query__rank(void);
 GwyModuleInfo* _gwy_module_query__rank_filter(void);
 GwyModuleInfo* _gwy_module_query__raveraging(void);
 GwyModuleInfo* _gwy_module_query__relate(void);
+GwyModuleInfo* _gwy_module_query__reorder(void);
 GwyModuleInfo* _gwy_module_query__resample(void);
 GwyModuleInfo* _gwy_module_query__residue_synth(void);
 GwyModuleInfo* _gwy_module_query__roddeposit_synth(void);
@@ -294,6 +295,7 @@ static const GwyModuleRecord modules[] = {
   { _gwy_module_query__rank_filter, "rank_filter", },
   { _gwy_module_query__raveraging, "raveraging", },
   { _gwy_module_query__relate, "relate", },
+  { _gwy_module_query__reorder, "reorder", },
   { _gwy_module_query__resample, "resample", },
   { _gwy_module_query__residue_synth, "residue_synth", },
   { _gwy_module_query__roddeposit_synth, "roddeposit_synth", },
