@@ -1,5 +1,5 @@
 /*
- *  $Id: sfunctions.c 27510 2025-03-10 15:33:48Z yeti-dn $
+ *  $Id: sfunctions.c 28495 2025-09-03 18:19:50Z francheu $
  *  Copyright (C) 2003-2023 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -177,7 +177,7 @@ static const GwyEnum quantities[] =  {
     { N_("HHCF"),                        GWY_SF_HHCF,                   },
     { N_("PSDF"),                        GWY_SF_PSDF,                   },
     { N_("Radial PSDF"),                 GWY_SF_RPSDF,                  },
-    { N_("Anguarly averaged PSDF"),      GWY_SF_IPSDF,                  },
+    { N_("Angularly averaged PSDF"),     GWY_SF_IPSDF,                  },
     { N_("Angular spectrum"),            GWY_SF_ANGSPEC,                },
     { N_("Radial ACF"),                  GWY_SF_RACF,                   },
     { N_("Minkowski volume"),            GWY_SF_MINKOWSKI_VOLUME,       },
