@@ -1,5 +1,5 @@
 /*
- *  $Id: volume_cluster.c 26744 2024-10-18 15:03:29Z yeti-dn $
+ *  $Id: volume_cluster.c 29036 2025-12-22 19:19:15Z yeti-dn $
  *  Copyright (C) 2014-2025 David Necas (Yeti), Petr Klapetek, Daniil Bratashov, Evgeniy Ryabov.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net, dn2010@gmail.com, k1u2r3ka@mail.ru.
  *
@@ -74,6 +74,11 @@ enum {
     PARAM_KMEDIANS_OUTLIERS_THRESHOLD,
     PARAM_DBSCAN_RANGE,
     PARAM_DBSCAN_NEIGHBORS,
+    PARAM_KNN_RANGE,
+    PARAM_KNN_MASK_A,
+    PARAM_KNN_MASK_B,
+    PARAM_KNN_K,
+    PARAM_KNN_METHOD,
     LABEL_RESULT,
 };
 
@@ -113,11 +118,20 @@ typedef enum {
     SCALING_AREA      = 5,
 } ClusterScaling;
 
+typedef enum {
+    KNN_K          = 0,
+    KNN_KW         = 1,
+    KNN_R          = 2,
+    KNN_RW         = 3,
+    KNN_NTYPES     = 4,
+} ClusterKNNType;
+
 typedef enum {
     METHOD_KMEANS      = 0,
     METHOD_KMEDIANS    = 1,
     METHOD_DBSCAN      = 2,
-    METHOD_NTYPES      = 3,
+    METHOD_KNN         = 3,
+    METHOD_NTYPES      = 4,
 } ClusterMethodType;
 
 typedef struct {
@@ -132,6 +146,8 @@ typedef struct {
     GwyDataField *mask;
     GwyGraphModel *gmodel;
     gdouble value;
+    GwyDataField *mask_a;
+    GwyDataField *mask_b;
 } ModuleArgs;
 
 typedef struct {
@@ -207,11 +223,13 @@ static void             sanitise_params        (ModuleArgs *args);
 DECLARE_METHOD(kmeans);
 DECLARE_METHOD(kmedians);
 DECLARE_METHOD(dbscan);
+DECLARE_METHOD(knn);
 
 static const ClusterMethod methods[] = {
-    { N_("K-means"),     METHOD_FUNCS(kmeans),    },
-    { N_("K-medians"),   METHOD_FUNCS(kmedians), },
-    { N_("DBSCAN"),      METHOD_FUNCS(dbscan), },
+    { N_("K-means"),             METHOD_FUNCS(kmeans),   },
+    { N_("K-medians"),           METHOD_FUNCS(kmedians), },
+    { N_("DBSCAN"),              METHOD_FUNCS(dbscan),   },
+    { N_("Nearest Neighbors"),   METHOD_FUNCS(knn),      },
 };
 
 static GwyModuleInfo module_info = {
@@ -312,9 +330,7 @@ define_module_params(void)
     gwy_param_def_add_boolean(paramdef, PARAM_OUTPUT_ORIGINAL_CENTERS, "output_original",
                               _("Output _non-processed centers"), FALSE);
     gwy_param_def_add_boolean(paramdef, PARAM_SHOW_CURVE, "show_curve", _("Sho_w data with result"), FALSE);
-
-
-    for (i = 0; i < G_N_ELEMENTS(methods); i++)
+      for (i = 0; i < G_N_ELEMENTS(methods); i++)
         methods[i].define_params(paramdef);
 
     return paramdef;
@@ -748,6 +764,12 @@ param_changed(ModuleGUI *gui, gint id)
            gwy_param_table_set_sensitive(gui->table_output, PARAM_OUTPUT_OUTLIERS, FALSE);
     }
 
+    if (id < 0 || id == PARAM_KNN_MASK_A)
+        args->mask_a = gwy_params_get_mask(params, PARAM_KNN_MASK_A);
+
+    if (id < 0 || id == PARAM_KNN_MASK_B)
+        args->mask_b = gwy_params_get_mask(params, PARAM_KNN_MASK_B);
+
     if (id < 0 || id == PARAM_OUTLIERS || id == PARAM_BACKGROUND || id == PARAM_SCALING || id == PARAM_TRANSFORM) {
         gwy_param_table_info_set_valuestr(gui->table_output, LABEL_RESULT, _("no clusters"));
         preprocess(args, GTK_WINDOW(gui->dialog));
@@ -774,7 +796,7 @@ update_graph_colors(GwyGraphModel *gmodel, GwyDataField *clusters, const guchar
         gcmodel = gwy_graph_model_get_curve(gmodel, i - 1);
         val = (i-cfrom)/(cto - cfrom);
         if (val>1) {
-            printf("gradient out of range: %d of nc %d, cfrom %g cto %g, value %g\n", i, nc, cfrom, cto, val);
+           // printf("gradient out of range: %d of nc %d, cfrom %g cto %g, value %g\n", i, nc, cfrom, cto, val);
             val = 1;
         }
         gwy_gradient_get_color(grad, val, &color);
@@ -1222,7 +1244,8 @@ scale(ModuleArgs *args, G_GNUC_UNUSED GtkWindow *wait_window)
 }
 
 static GwyGraphModel*
-create_graph(GwyBrick *brick, GwyDataLine *calibration, const gdouble *centers, gint nclusters, gint *npix)
+create_graph(GwyBrick *brick, G_GNUC_UNUSED GwyDataLine *calibration,
+             const gdouble *centers, gint nclusters, gint *npix)
 {
     gint xres = gwy_brick_get_xres(brick);
     gdouble xreal = gwy_brick_get_xreal(brick);
@@ -1235,19 +1258,22 @@ create_graph(GwyBrick *brick, GwyDataLine *calibration, const gdouble *centers,
 
     xdata = g_new(gdouble, xres);
     ydata = g_new(gdouble, xres);
-    if (calibration) {
-        gwy_assign(xdata, gwy_data_line_get_data(calibration), xres);
-        siunit = gwy_data_line_get_si_unit_y(calibration);
-    }
-    else {
+//    if (calibration) {
+//        gwy_assign(xdata, gwy_data_line_get_data(calibration), xres);
+//        siunit = gwy_data_line_get_si_unit_y(calibration);
+
+//    }
+//    else {
+      //printf("xreal %g\n", xreal);
         gwy_math_linspace(xdata, xres, xoffset, xreal/xres);
         siunit = gwy_brick_get_si_unit_x(brick);
-    }
+//    }
 
     for (c = 0; c < nclusters; c++) {
         if (npix[c]==0)
             continue;
         gwy_assign(ydata, centers + c*xres, xres);
+        //printf("%d npix %d %g %g %g %g\n", c, npix[c], ydata[0], ydata[1], ydata[2], ydata[3]);
         gcmodel = gwy_graph_curve_model_new();
         gwy_graph_curve_model_set_data(gcmodel, xdata, ydata, xres);
         g_object_set(gcmodel,
@@ -2186,4 +2212,282 @@ fail:
 }
 
 
+/************************************************************************************************************
+ *
+ * KNN
+ *
+ ************************************************************************************************************/
+static void
+define_params_knn(GwyParamDef *paramdef)
+{
+    static const GwyEnum knns[] = {
+        { N_("Number"),              KNN_K,   },
+        { N_("Number (weighted) "),  KNN_KW,  },
+        { N_("Distance"),            KNN_R,   },
+        { N_("Distance (weighted)"), KNN_RW,  },
+    };
+    gwy_param_def_add_image_id(paramdef, PARAM_KNN_MASK_A, "mask_a", _("Mask A"));
+    gwy_param_def_add_image_id(paramdef, PARAM_KNN_MASK_B, "mask_b", _("Mask B"));
+    gwy_param_def_add_percentage(paramdef, PARAM_KNN_RANGE, "knn/range", _("_Range factor"), 0.1);
+    gwy_param_def_add_gwyenum(paramdef, PARAM_KNN_METHOD, "knn/method", _("_Criterion"),
+                              knns, G_N_ELEMENTS(knns), KNN_K);
+   
+}
+
+static gboolean
+mask_filter(GwyContainer *data, gint id, gpointer user_data)
+{
+    GwyDataField *preview = (GwyDataField*)user_data;
+    GwyDataField *field = gwy_container_get_object(data, gwy_app_get_data_key_for_id(id));
+    GwyDataField *mask = NULL;
+
+    if (!gwy_container_gis_object(data, gwy_app_get_mask_key_for_id(id), &mask))
+        return FALSE;
+
+    return !gwy_data_field_check_compatibility(preview, field,
+                                               GWY_DATA_COMPATIBILITY_LATERAL);
+}
+
+static void
+append_gui_knn(ModuleGUI *gui)
+{
+    GwyParamTable *table = gui->table_methodparam[METHOD_KNN];
+
+    gwy_param_table_append_combo(table, PARAM_KNN_METHOD);
+
+    gwy_param_table_append_image_id(table, PARAM_KNN_MASK_A);
+    gwy_param_table_data_id_set_filter(table, PARAM_KNN_MASK_A, mask_filter, gui->showfield, NULL);
+
+    gwy_param_table_append_image_id(table, PARAM_KNN_MASK_B);
+    gwy_param_table_data_id_set_filter(table, PARAM_KNN_MASK_B, mask_filter, gui->showfield, NULL);
+
+    gwy_param_table_append_slider(table, PARAM_KNN_RANGE);
+}
+
+static gdouble
+cdistance(const gdouble *data, gint xres, gint yres, gint k, gint j, gint nk, gint nj)
+{
+    gint i;
+    gdouble d, sd = 0;
+    for (i = 0; i < xres; i++) {
+        d = data[k*xres*yres + j*xres + i] - data[nk*xres*yres + nj*xres + i];
+        sd += d*d;
+    }
+    return sd;
+}
+
+static gboolean
+cluster_knn(ModuleArgs *args, GtkWindow *wait_window)
+{
+    GwyParams *params = args->params;
+    GwyBrick *brick = args->prepbrick;
+
+    gdouble range = gwy_params_get_double(params, PARAM_KNN_RANGE);
+    gint method = gwy_params_get_int(params, PARAM_KNN_METHOD);
+    gint xres = gwy_brick_get_xres(brick), yres = gwy_brick_get_yres(brick), zres = gwy_brick_get_zres(brick);
+    GwyDataField *clusters = NULL, *errormap = NULL;
+    GwyDataField *mask = args->mask;
+    GwyDataField *mask_a = args->mask_a;
+    GwyDataField *mask_b = args->mask_b;
+    const gdouble *data;
+    gdouble *centers;
+    gboolean ok = FALSE, cancelled = FALSE, done;
+    gint i, j, k, m, nk, nj, npos, nclusters, nvalid;
+    guint *nblisti;
+    gint *npix, *nnpix, nmasked;
+    gdouble *nblistd, *labeldata, *mcdist, *maskdata, *madata, *mbdata, *mc;
+    gdouble score, nma, nmb;
+
+    clusters = gwy_data_field_new(yres, zres, gwy_brick_get_xreal(args->brick), gwy_brick_get_yreal(args->brick), TRUE);
+    gwy_data_field_set_xoffset(clusters, gwy_brick_get_xoffset(args->brick));
+    gwy_data_field_set_yoffset(clusters, gwy_brick_get_yoffset(args->brick));
+    gwy_si_unit_assign(gwy_data_field_get_si_unit_xy(clusters), gwy_brick_get_si_unit_x(args->brick));
+
+    gwy_app_wait_start(wait_window, _("Initializing..."));
+
+    data = gwy_brick_get_data_const(brick);
+    npos = yres*zres;
+
+    gwy_data_field_fill(clusters, -1);             //unassigned
+    labeldata = gwy_data_field_get_data(clusters);
+
+    nblisti = g_new(guint, npos*sizeof(gint));    //neighbors list
+    nblistd = g_new(gdouble, npos*sizeof(gdouble));
+    mc = g_new0(gdouble, xres*sizeof(gdouble));  //mean curve
+
+    errormap = gwy_data_field_new_alike(clusters, TRUE);
+    mcdist = gwy_data_field_get_data(errormap);
+    maskdata = gwy_data_field_get_data(mask);
+    madata = gwy_data_field_get_data(mask_a);
+    mbdata = gwy_data_field_get_data(mask_b);
+
+    //get some normalisation for range: a mean curve and variance of all the curves from it
+    //mark points that are under mask, not to be used for clustering
+    nvalid = 0;
+    nmasked = 0;
+    for (k = 0; k < zres; k++) {
+        for (j = 0; j < yres; j++) {
+            if (maskdata[k*yres + j] != 0)
+                continue;
+            if (madata[k*yres + j] != 0 || mbdata[k*yres + j] != 0)
+                nmasked++;
+             for (i = 0; i < xres; i++) {
+                mc[i] += data[k*xres*yres + j*xres + i];
+            }
+            nvalid++;
+        }
+    }
+    for (i = 0; i < xres; i++) {
+        mc[i] /= nvalid;
+    }
+    for (k = 0; k < zres; k++) {
+        for (j = 0; j < yres; j++) {
+            if (maskdata[k*yres + j] != 0)
+                labeldata[k*yres + j] = -3; //ignore this point while clustering
+            else {
+                for (i = 0; i < xres; i++) {
+                    mcdist[k*yres + j] +=
+                        (data[k*xres*yres + j*xres + i] - mc[i])*(data[k*xres*yres + j*xres + i] - mc[i]);
+                }
+            }
+        }
+    }
+    for (k = 0; k < zres; k++) {
+        for (j = 0; j < yres; j++) {
+            mcdist[k*yres + j] = sqrt(mcdist[k*yres + j]);
+        }
+    }
+
+    gwy_data_field_data_changed(errormap);
+
+    if (method == KNN_K || method == KNN_KW) //full range is defined by number of train data
+        range *= nmasked;
+    else { //full range is defined by distance between the curves
+        range = 10*range*gwy_data_field_area_get_rms_mask(errormap, mask, GWY_MASK_EXCLUDE, 0, 0, yres, zres);
+        range *= range; //we work with squared range when applying distance threshold
+    }
+
+    //perform clustering
+    if (!(gwy_app_wait_set_message(_("Clustering...")) && gwy_app_wait_set_fraction(0.0))) {
+        cancelled = TRUE;
+        goto fail;
+    }
+
+    for (k = 0; k < zres; k++) {
+        for (j = 0; j < yres; j++) {
+            //keep masked areas as clusters
+            if (madata[k*yres + j] && !mbdata[k*yres + j]) {
+                labeldata[k*yres + j] = 0;
+                continue;
+            }
+            if (mbdata[k*yres + j] && !madata[k*yres + j]) {
+                labeldata[k*yres + j] = 1;
+                continue;
+            }
+
+            //ignore masked data
+            if (maskdata[k*yres + j]) {
+                labeldata[k*yres + j] = -1;
+                continue;
+            }
+
+            //go through all the curves, where is mask_a or mask_b, add it
+            //to the list of neighbors and distances
+            
+            m = 0;
+            for (nk = 0; nk < zres; nk++) {
+                for (nj = 0; nj < yres; nj++) {
+                    //ignore masked data and myself
+                   if (maskdata[nk*yres + nj] || (nk == k && nj == j))
+                        continue;
+
+                    if (madata[nk*yres + nj] || mbdata[nk*yres + nj]) {
+                        nblisti[m] = nk*yres + nj;
+                        nblistd[m] = cdistance(data, xres, yres, k, j, nk, nj);
+                        m++;
+                    }
+                }
+            }
+            
+            if (m == 0)
+                labeldata[k*yres + j] = 0;
+            else {
+                gwy_math_sort_with_index(m, nblistd, nblisti);
+                //for (i=0; i<m; i++) printf("%d %g  a %g b %g\n", 
+                //i, nblistd[i], madata[nblisti[i]], mbdata[nblisti[i]]);
+
+                nma = nmb = 0;
+                i = 0;
+                done = FALSE;
+                do {
+                    if (method == KNN_KW || method == KNN_RW)
+                        score = 1.0/(sqrt(nblistd[i]) + 1e-15);
+                    else
+                        score = 1.0;
+
+                    if (madata[nblisti[i]])
+                        nma += score;
+                    if (mbdata[nblisti[i]])
+                        nmb += score;
+
+                    if (method == KNN_K || method == KNN_KW) {
+                        if (i > range)
+                            done = TRUE; 
+                    } else {
+                        if (nblistd[i] > range)
+                            done = TRUE;
+                    }
+
+                    i++;
+                } while (i<m && !done);
+               
+                if (nma > nmb)
+                    labeldata[k*yres + j] = 0;
+                else if (nmb > nma)
+                    labeldata[k*yres + j] = 1;
+                else {
+                    labeldata[k*yres + j] = -1;
+                }
+            }
+        }
+        if (!gwy_app_wait_set_fraction((gdouble)j/yres)) {
+             cancelled = TRUE;
+             break;
+        }
+    }
+    gwy_app_wait_finish();
+    if (cancelled) {
+        goto fail;
+    }
+
+    //create cluster centers
+    nclusters = 2;
+    centers = g_new0(gdouble, nclusters*xres);
+    npix = g_new0(gint, nclusters);
+    create_cluster_centers(brick, clusters, centers, nclusters, npix);
+    create_error_map(brick, errormap, clusters, centers, nclusters);
+
+    nnpix = g_new0(gint, nclusters);
+    resort_clusters_by_size(clusters, centers, nclusters, xres, npix, nnpix);
+
+    gwy_data_field_add(clusters, 1); //shift clusters to start at 1???????????????????
+
+    args->result = g_object_ref(clusters);
+    args->errormap = g_object_ref(errormap);
+    args->gmodel = create_graph(brick, gwy_brick_get_zcalibration(args->brick), centers, nclusters, nnpix);
+    ok = TRUE;
+
+    g_free(centers);
+    g_free(npix);
+
+fail:
+    GWY_OBJECT_UNREF(errormap);
+    GWY_OBJECT_UNREF(clusters);
+    g_free(nblisti);
+    g_free(nblistd);
+    g_free(mc);
+
+    return ok;
+}
+
 /* vim: set cin columns=120 tw=118 et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
