@@ -1,5 +1,5 @@
 /*
- *  $Id: volumeops.c 26421 2024-06-28 11:19:22Z yeti-dn $
+ *  $Id: volumeops.c 28692 2025-10-21 14:05:58Z klapetek $
  *  Copyright (C) 2003 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -32,11 +32,11 @@ static void     extract_preview(GwyContainer *data,
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
     &module_register,
-    N_("Inverts value in volume data"),
-    "Yeti <yeti@gwyddion.net>",
-    "1.1",
-    "David Nečas (Yeti)",
-    "2017",
+    N_("Extracts preview"),
+    "Petr Klapetek <klapetek@gwyddion.net>",
+    "1.0",
+    "David Nečas (Yeti) & Petr Klapetek",
+    "2025",
 };
 
 GWY_MODULE_QUERY2(module_info, volumeops)
