@@ -21,6 +21,8 @@ GwyModuleInfo* _gwy_module_query__xyz_raster(void);
 GwyModuleInfo* _gwy_module_query__xyz_reduce(void);
 GwyModuleInfo* _gwy_module_query__xyz_split(void);
 GwyModuleInfo* _gwy_module_query__xyz_tcut(void);
+GwyModuleInfo* _gwy_module_query__xyz_tstep(void);
+GwyModuleInfo* _gwy_module_query__xyz_volumize(void);
 GwyModuleInfo* _gwy_module_query__xyz_zdrift(void);
 
 static const GwyModuleRecord modules[] = {
@@ -33,6 +35,8 @@ static const GwyModuleRecord modules[] = {
   { _gwy_module_query__xyz_reduce, "xyz_reduce", },
   { _gwy_module_query__xyz_split, "xyz_split", },
   { _gwy_module_query__xyz_tcut, "xyz_tcut", },
+  { _gwy_module_query__xyz_tstep, "xyz_tstep", },
+  { _gwy_module_query__xyz_volumize, "xyz_volumize", },
   { _gwy_module_query__xyz_zdrift, "xyz_zdrift", },
   { NULL, NULL, },
 };
