@@ -1,5 +1,5 @@
 /*
- *  $Id: xyz_tcut.c 26355 2024-05-21 08:23:55Z yeti-dn $
+ *  $Id: xyz_tcut.c 28891 2025-11-20 12:02:29Z klapetek $
  *  Copyright (C) 2016-2023 David Necas (Yeti).
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
@@ -27,7 +27,8 @@
 #define RUN_MODES (GWY_RUN_INTERACTIVE)
 
 enum {
-    PREVIEW_SIZE = 360,
+    PREVIEW_WIDTH = 500,
+    PREVIEW_HEIGHT = 300,
 };
 
 
@@ -158,6 +159,8 @@ xyztcut(GwyContainer *data, GwyRunType runtype)
     if (gwy_container_gis_string(data, gwy_app_get_surface_palette_key_for_id(id), &gradient))
         gwy_container_set_const_string(data, gwy_app_get_surface_palette_key_for_id(newid), gradient);
 
+    gwy_app_xyz_log_add(data, -1, newid, "xyz::xyz_tcut", NULL);
+
     g_object_unref(args.result);
 
 end:
@@ -190,7 +193,7 @@ run_gui(ModuleArgs *args)
 
     graph = gwy_graph_new(gui.gmodel);
     gwy_graph_enable_user_input(GWY_GRAPH(graph), FALSE);
-    gtk_widget_set_size_request(graph, PREVIEW_SIZE, PREVIEW_SIZE);
+    gtk_widget_set_size_request(graph, PREVIEW_WIDTH, PREVIEW_HEIGHT);
     gwy_dialog_add_content(dialog, graph, TRUE, TRUE, 0);
 
     area = gwy_graph_get_area(GWY_GRAPH(graph));
